@@ -58,7 +58,6 @@ Usage: zxencdectest [options] <foo.xml >reencoded-foo.xml\n\
                http://www.aet.tu-cottbus.de/personen/jaenicke/postfix_tls/prngd.html\n\
   -rand PATH   Location of random number seed file. On Solaris EGD is used.\n\
                On Linux the default is /dev/urandom. See RFC1750.\n\
-  -so PATH     File to write schema order encoding in\n\
   -wo PATH     File to write wire order encoding in\n\
   -v           Verbose messages.\n\
   -q           Be extra quiet.\n\
@@ -70,22 +69,22 @@ Usage: zxencdectest [options] <foo.xml >reencoded-foo.xml\n\
 #define DIE(reason) MB fprintf(stderr, "%s\n", reason); exit(2); ME
 char buf[256*1024];
 
-/* Called by:  opt */
+/* Called by:  opt x2 */
 void test_ibm_cert_problem()
 {
-  int got_all, len_so;
+  int len, got_all;
   zxid_conf* cf;
   struct zx_root_s* r;
   struct zx_sp_LogoutRequest_s* req;
 
-  len_so = read_all_fd(0, buf, sizeof(buf)-1, &got_all);
+  len = read_all_fd(0, buf, sizeof(buf)-1, &got_all);
   if (got_all <= 0) DIE("Missing data");
   buf[got_all] = 0;
 
   /* IBM padding debug */
   cf = zxid_new_conf("/var/zxid/");
   r = zx_dec_zx_root(cf->ctx, got_all, buf, "zxencdectest");
-  if (!r)
+  if (!r || !r->Envelope || r->Envelope->Body || r->Envelope->Body->LogoutRequest)
     DIE("Decode failure");
 
 #if 1
@@ -99,7 +98,7 @@ void test_ibm_cert_problem()
   printf("r1 nid(%.*s)\n", ZX_GET_CONTENT_LEN(req->NameID), ZX_GET_CONTENT_S(req->NameID));
 }
 
-/* Called by:  opt */
+/* Called by:  opt x2 */
 void test_ibm_cert_problem_enc_dec()
 {
   zxid_conf* cf;
@@ -110,16 +109,16 @@ void test_ibm_cert_problem_enc_dec()
   cf = zxid_new_conf("/var/zxid/");
 
   nameid = zx_NEW_sa_NameID(cf->ctx,0);
-  nameid->Format = zx_ref_attr(cf->ctx, zx_Format_ATTR, "persistent");
-  nameid->NameQualifier = zx_ref_attr(cf->ctx, zx_NameQualifier_ATTR, "ibmidp");
-  /*nameid->SPNameQualifier = zx_ref_attr(cf->ctx, zx_SPNameQualifier_ATTR, spqual);*/
+  nameid->Format = zx_ref_attr(cf->ctx, &nameid->gg, zx_Format_ATTR, "persistent");
+  nameid->NameQualifier = zx_ref_attr(cf->ctx, &nameid->gg, zx_NameQualifier_ATTR, "ibmidp");
+  /*nameid->SPNameQualifier = zx_ref_attr(cf->ctx, &nameid->gg, zx_SPNameQualifier_ATTR, spqual);*/
   zx_add_content(cf->ctx, &nameid->gg, zx_ref_str(cf->ctx, "a-persistent-nid"));
 
 #if 0
   cf->enc_pkey = zxid_read_private_key(cf, "sym-idp-enc.pem");
 #else
   cf->enc_pkey = zxid_read_private_key(cf, "ibm-idp-enc.pem");
-  idp_meta = zxid_get_ent_from_file(cf, "N9zsU-AwbI1O-U3mvjLmOALtbtU"); /* IBMIdP */
+  idp_meta = zxid_get_ent_file(cf, "N9zsU-AwbI1O-U3mvjLmOALtbtU"); /* IBMIdP */
 #endif
   
   req = zxid_mk_logout(cf, nameid, 0, idp_meta);  
@@ -127,6 +126,29 @@ void test_ibm_cert_problem_enc_dec()
   printf("r2 nid(%.*s) should be(a-persistent-nid)\n", ZX_GET_CONTENT_LEN(req->NameID), ZX_GET_CONTENT_S(req->NameID));
 }
 
+/* Called by:  opt */
+void so_enc_dec()
+{
+  zxid_conf* cf;
+  struct zx_sp_Status_s* st;
+  struct zx_str* ss;
+  cf = zxid_new_conf("/var/zxid/");
+  st = zxid_mk_Status(cf, 0, "SC1", "SC2", "MESSAGE");
+  ss = zx_EASY_ENC_elem(cf->ctx, &st->gg);
+  printf("%.*s", ss->len, ss->s);
+}
+
+void attribute_sort_test()
+{
+  zxid_conf* cf;
+  struct zx_xasp_XACMLAuthzDecisionQuery_s* q;
+  struct zx_str* ss;
+  cf = zxid_new_conf("/var/zxid/");
+  q = zxid_mk_az(cf, 0, 0, 0, 0);
+  ss = zx_EASY_ENC_elem(cf->ctx, &q->gg);
+  printf("%.*s", ss->len, ss->s);
+}
+
 int afr_buf_size = 0;
 int verbose = 1;
 extern int debug;
@@ -141,10 +163,9 @@ char* egd_path;
 char  symmetric_key[1024];
 int symmetric_key_len;
 int n_iter = 1;
-char* so_path = 0;
 char* wo_path = 0;
 
-/* Called by:  main x8, zxcall_main, zxcot_main */
+/* Called by:  main x8, zxcall_main, zxcot_main, zxdecode_main */
 void opt(int* argc, char*** argv, char*** env)
 {
   if (*argc < 1) goto argerr;
@@ -219,6 +240,8 @@ void opt(int* argc, char*** argv, char*** env)
 	switch (atoi((*argv)[0])) {
 	case 1: test_ibm_cert_problem(); break;
 	case 2: test_ibm_cert_problem_enc_dec(); break;
+	case 3: so_enc_dec(); break;
+	case 4: attribute_sort_test(); break;
 	}
 	exit(0);
 
@@ -272,16 +295,6 @@ void opt(int* argc, char*** argv, char*** env)
       }
       break;
 
-    case 's':
-      switch ((*argv)[0][2]) {
-      case 'o': if ((*argv)[0][3]) break;
-	++(*argv); --(*argc);
-	if (!(*argc)) break;
-	so_path = (*argv)[0];
-	continue;
-      }
-      break;
-
     case 'w':
       switch ((*argv)[0][2]) {
       case 'o': if ((*argv)[0][3]) break;
@@ -350,14 +363,12 @@ int main(int argc, char** argv, char** env)
 {
   struct zx_ctx ctx;
   struct zx_root_s* r;
-  int got_all, len_so, len_wo;
-  char so_out[256*1024];
-  char* so_p;
+  int got_all, len_wo;
   char wo_out[256*1024];
   char* wo_p;
   opt(&argc, &argv, &env);
   
-  len_so = read_all_fd(0, buf, sizeof(buf)-1, &got_all);
+  len_wo = read_all_fd(0, buf, sizeof(buf)-1, &got_all);
   if (got_all <= 0) DIE("Missing data");
   buf[got_all] = 0;
 
@@ -377,14 +388,6 @@ int main(int argc, char** argv, char** env)
     if (!wo_p)
       DIE("encoding error");
 
-    len_so = zx_LEN_SO_root(&ctx, r);
-    D("Enc so len %d chars", len_so);
-
-    ctx.bas = so_out;
-    so_p = zx_ENC_SO_root(&ctx, r, so_out);
-    if (!so_p)
-      DIE("encoding error");
-
     zx_free_elem(&ctx, &r->gg, 0);
   }
 
@@ -394,19 +397,10 @@ int main(int argc, char** argv, char** env)
   if (memcmp(buf, wo_out, MIN(got_all, len_wo)))
     printf("Original and WO differ.\n");
 
-  if (memcmp(so_out, wo_out, MIN(len_so, len_wo)))
-    printf("SO and WO differ.\n");
-
-  if (so_p - so_out != len_so)
-    ERR("SO encode length mismatch %d vs. %d (len)", so_p - so_out, len_so);
-  printf("Re-encoded result SO (len=%d):\n%.*s\n\n", len_so, len_so, so_out);
-
   if (wo_p - wo_out != len_wo)
     ERR("WO encode length mismatch %d vs %d (len)", wo_p - wo_out, len_wo);
   printf("Re-encoded result WO (len=%d):\n%.*s\n\n", len_wo, len_wo, wo_out);
 
-  if (so_path)
-    write_all_path_fmt("SO", sizeof(buf), buf, "%s", so_path, 0, "%.*s", len_so, so_out);
   if (wo_path)
     write_all_path_fmt("WO", sizeof(buf), buf, "%s", wo_path, 0, "%.*s", len_wo, wo_out);
   return 0;
