@@ -31,6 +31,7 @@
  * that was created when token for accessing discovery was issued.
  * Returns 1 on success, 0 on failure. */
 
+/* Called by:  zxid_di_query, zxid_imreq, zxid_ps_addent_invite, zxid_ps_resolv_id, zxid_ssos_anreq */
 int zxid_idp_map_nid2uid(zxid_conf* cf, int len, char* uid, zxid_a7n* a7n, struct zx_lu_Status_s** stp, zxid_nid** nameidp)
 {
   zxid_nid* nameid;
@@ -40,26 +41,26 @@ int zxid_idp_map_nid2uid(zxid_conf* cf, int len, char* uid, zxid_a7n* a7n, struc
   if (!a7n || !a7n->Subject) {
     ERR("Malformed Assertion(%p): Subject missing.", a7n);
     if (stp)
-      *stp = zxid_mk_lu_Status(cf, "Fail", 0, 0, 0);
+      *stp = zxid_mk_lu_Status(cf, 0, "Fail", 0, 0, 0);
     return 0;
   }
 
   nameid = zxid_decrypt_nameid(cf, a7n->Subject->NameID, a7n->Subject->EncryptedID);
   if (nameidp)
     *nameidp = nameid;
-  affil = nameid->SPNameQualifier ? &nameid->SPNameQualifier->g : zxid_my_entity_id(cf);
+  affil = nameid->SPNameQualifier ? &nameid->SPNameQualifier->g : zxid_my_ent_id(cf);
   zxid_nice_sha1(cf, sp_name_buf, sizeof(sp_name_buf), affil, affil, 7);
   len = read_all(len-1, uid, "idp_map_nid2uid", 1, "%s" ZXID_NID_DIR "%s/%.*s", cf->path, sp_name_buf, ZX_GET_CONTENT_LEN(nameid), ZX_GET_CONTENT_S(nameid));
   if (!len) {
     ERR("Can not find reverse mapping for SP,SHA1(%s) nid(%.*s)", sp_name_buf, ZX_GET_CONTENT_LEN(nameid), ZX_GET_CONTENT_S(nameid));
     if (stp)
-      *stp = zxid_mk_lu_Status(cf, "Fail", 0, 0, 0);
+      *stp = zxid_mk_lu_Status(cf, 0, "Fail", 0, 0, 0);
     return 0;
   }
   return 1;
 }
 
-/*() Server side  Discovery Service Query processing. See also zxid_gen_bootstraps() */
+/*() Server side Discovery Service Query processing. See also zxid_gen_bootstraps() */
 
 /* Called by:  zxid_sp_soap_dispatch */
 struct zx_di_QueryResponse_s* zxid_di_query(zxid_conf* cf, zxid_a7n* a7n, struct zx_di_Query_s* req, struct zx_str* issuer)
@@ -112,7 +113,7 @@ struct zx_di_QueryResponse_s* zxid_di_query(zxid_conf* cf, zxid_a7n* a7n, struct
     if (!dir) {
       perror("opendir to find service metadata");
       ERR("Opening service metadata directory failed path(%s)", mdpath);
-      resp->Status = zxid_mk_lu_Status(cf, "Fail", 0, 0, 0);
+      resp->Status = zxid_mk_lu_Status(cf, &resp->gg, "Fail", 0, 0, 0);
       D_DEDENT("di_query: ");
       return resp;
     }
@@ -261,14 +262,15 @@ struct zx_di_QueryResponse_s* zxid_di_query(zxid_conf* cf, zxid_a7n* a7n, struct
 #endif
       ++n_discovered;
       D("%d: DISCOVERED EPR url(%.*s)", n_discovered, addr->len, addr->s);
-      logop = zxid_add_fed_tok_to_epr(cf, epr, uid, 1);
+      logop = zxid_add_fed_tok2epr(cf, epr, uid, 1);
       if (!logop) {
 	ZX_FREE(cf->ctx, epr_buf);
 	goto next_file;
       }
 
-      epr->gg.g.n = (void*)resp->EndpointReference;
-      resp->EndpointReference = epr;
+      zx_add_kid(&resp->gg, &epr->gg);
+      if (!resp->EndpointReference)
+	resp->EndpointReference = epr;
 
       zxlog(cf, 0, 0, 0, issuer, 0, &a7n->ID->g, ZX_GET_CONTENT(nameid), "N", "K", logop, uid, "");
 
@@ -288,7 +290,7 @@ next_file:
   ss = ZX_GET_CONTENT(req->RequestedService->ServiceType);
   D("TOTAL discovered %d svctype(%.*s)", n_discovered, ss?ss->len:0, ss?ss->s:"");
   zxlog(cf, 0, 0, 0, issuer, 0, &a7n->ID->g, ZX_GET_CONTENT(nameid), "N", "K", "DIOK", 0, "%.*s n=%d", ss?ss->len:1, ss?ss->s:"-", n_discovered);
-  resp->Status = zxid_mk_lu_Status(cf, "OK", 0, 0, 0);
+  resp->Status = zxid_mk_lu_Status(cf, &resp->gg, "OK", 0, 0, 0);  /* last is first */
   D_DEDENT("di_query: ");
   return resp;
 }
