@@ -28,9 +28,6 @@
 #include "c/zx-const.h"
 #include "c/zx-paos-data.h"
 
-#define zx_e_actor_ATTR (zx_e_NS|zx_actor_ATTR)
-#define zx_e_mustUnderstand_ATTR (zx_e_NS|zx_mustUnderstand_ATTR)
-
 extern char **environ;
 
 /*() Generate SOAP headers for use with PAOS carried SAML2 ECP profile AuthnRequest.
@@ -41,11 +38,11 @@ extern char **environ;
 struct zx_paos_Request_s* zxid_mk_paos_Request_hdr(zxid_conf* cf)
 {
   struct zx_paos_Request_s* hdr= zx_NEW_paos_Request(cf->ctx,0);
+  hdr->mustUnderstand = zx_ref_attr(cf->ctx, &hdr->gg, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
+  hdr->actor   = zx_ref_attr(cf->ctx, &hdr->gg, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
+  hdr->service = zx_ref_attr(cf->ctx, &hdr->gg, zx_service_ATTR, SAML2_SSO_ECP);
+  hdr->responseConsumerURL = zx_attrf(cf->ctx, &hdr->gg, zx_responseConsumerURL_ATTR, "%s?o=P", cf->url);
   /*hdr->messageID = zx_ref_str(cf->ctx, "1"); OPTIONAL */
-  hdr->mustUnderstand = zx_ref_attr(cf->ctx, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
-  hdr->actor = zx_ref_attr(cf->ctx, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
-  hdr->service = zx_ref_attr(cf->ctx, zx_service_ATTR, SAML2_SSO_ECP);
-  hdr->responseConsumerURL = zx_attrf(cf->ctx, zx_responseConsumerURL_ATTR, "%s?o=P", cf->url);
   return hdr;
 }
 
@@ -85,8 +82,8 @@ struct zx_sp_IDPList_s* zxid_mk_idp_list(zxid_conf* cf, char* binding)
     }
     
     idp_list->IDPEntry = idp_entry = zx_NEW_sp_IDPEntry(cf->ctx, &idp_list->gg);
-    idp_entry->ProviderID = zx_ref_attr(cf->ctx, zx_ProviderID_ATTR, idp->eid);
-    idp_entry->Name = zx_ref_attr(cf->ctx, zx_Name_ATTR, idp->dpy_name);
+    idp_entry->ProviderID = zx_ref_attr(cf->ctx, &idp_entry->gg, zx_ProviderID_ATTR, idp->eid);
+    idp_entry->Name = zx_ref_attr(cf->ctx, &idp_entry->gg, zx_Name_ATTR, idp->dpy_name);
     idp_entry->Loc = sso_svc->Location;
   }
   return idp_list;
@@ -100,11 +97,11 @@ struct zx_sp_IDPList_s* zxid_mk_idp_list(zxid_conf* cf, char* binding)
 struct zx_ecp_Request_s* zxid_mk_ecp_Request_hdr(zxid_conf* cf)
 {
   struct zx_ecp_Request_s* hdr= zx_NEW_ecp_Request(cf->ctx,0);
-  hdr->mustUnderstand = zx_ref_attr(cf->ctx, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
-  hdr->actor = zx_ref_attr(cf->ctx, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
-  /*hdr->IsPassive = zx_ref_attr(cf->ctx, zx_IsPassive_ATTR, ZXID_TRUE);  OPTIONAL, default=? */
-  hdr->ProviderName = zxid_my_entity_id_attr(cf, zx_ProviderName_ATTR);  /* *** Friendly name? */
-  hdr->Issuer = zxid_my_issuer(cf);
+  hdr->mustUnderstand = zx_ref_attr(cf->ctx, &hdr->gg, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
+  hdr->actor = zx_ref_attr(cf->ctx, &hdr->gg, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
+  /*hdr->IsPassive = zx_ref_attr(cf->ctx, &hdr->gg, zx_IsPassive_ATTR, ZXID_TRUE);  OPTIONAL, default=? */
+  hdr->ProviderName = zxid_my_ent_id_attr(cf, &hdr->gg, zx_ProviderName_ATTR);  /* *** Friendly name? */
+  hdr->Issuer = zxid_my_issuer(cf, &hdr->gg);
   hdr->IDPList = zxid_mk_idp_list(cf, SAML2_SOAP);
   return hdr;
 }
@@ -156,7 +153,7 @@ struct zx_str* zxid_lecp_check(zxid_conf* cf, zxid_cgi* cgi)
   se->Header = zx_NEW_e_Header(cf->ctx, &se->gg);
   se->Header->Request = zxid_mk_paos_Request_hdr(cf);
   se->Header->ecp_Request = zxid_mk_ecp_Request_hdr(cf);
-  env = zx_EASY_ENC_SO_e_Envelope(cf->ctx, se);
+  env = zx_EASY_ENC_elem(cf->ctx, &se->gg);
   req = zx_strf(cf->ctx,
 		"Cache-Control: no-cache, no-store, must-revalidate, private" CRLF
 		"Pragma: no-cache" CRLF
