@@ -64,7 +64,7 @@ char* zxid_version_str()
  *     bits should be multiple of 24 (3 bytes expands to 4 safe base64 chars)
  * return:: The identifier as zx_str. Caller should eventually free this memory.
  */
-/* Called by:  zxid_check_fed, zxid_di_query, zxid_mk_a7n, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_dap_query_item, zxid_mk_dap_resquery, zxid_mk_dap_subscription, zxid_mk_dap_test_item, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp, zxid_mk_subj, zxid_mk_transient_nid, zxid_put_ses, zxid_pw_authn, zxid_wsc_prep_secmech, zxid_wsf_decor */
+/* Called by:  zxid_check_fed, zxid_mk_subj, zxid_mk_transient_nid, zxid_ps_addent_invite x3, zxid_ps_resolv_id, zxid_put_ses, zxid_pw_authn, zxid_ssos_anreq, zxid_wsc_prep_secmech, zxid_wsf_decor */
 struct zx_str* zxid_mk_id(zxid_conf* cf, char* prefix, int bits)
 {
   char bit_buf[ZXID_ID_MAX_BITS/8];
@@ -79,7 +79,8 @@ struct zx_str* zxid_mk_id(zxid_conf* cf, char* prefix, int bits)
   return zx_strf(cf->ctx, "%s%.*s", prefix?prefix:"", p-base64_buf, base64_buf);
 }
 
-struct zx_attr_s* zxid_mk_id_attr(zxid_conf* cf, int tok, char* prefix, int bits)
+/* Called by:  zxid_mk_a7n, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_dap_query_item, zxid_mk_dap_resquery, zxid_mk_dap_subscription, zxid_mk_dap_test_item, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp */
+struct zx_attr_s* zxid_mk_id_attr(zxid_conf* cf, struct zx_elem_s* father, int tok, char* prefix, int bits)
 {
   char bit_buf[ZXID_ID_MAX_BITS/8];
   char base64_buf[ZXID_ID_MAX_BITS/6 + 1];
@@ -90,7 +91,7 @@ struct zx_attr_s* zxid_mk_id_attr(zxid_conf* cf, int tok, char* prefix, int bits
   }
   zx_rand(bit_buf, bits >> 3);
   p = base64_fancy_raw(bit_buf, bits >> 3, base64_buf, safe_basis_64, 1<<31, 0, 0, '.');
-  return zx_attrf(cf->ctx, tok, "%s%.*s", prefix?prefix:"", p-base64_buf, base64_buf);
+  return zx_attrf(cf->ctx, father, tok, "%s%.*s", prefix?prefix:"", p-base64_buf, base64_buf);
 }
 
 /*() Format a date-time string as usually used in XML, SAML, and Liberty. Apparently
@@ -100,7 +101,7 @@ struct zx_attr_s* zxid_mk_id_attr(zxid_conf* cf, int tok, char* prefix, int bits
  * accept the without milliseconds form. You can change the format at compile time
  * by editing zxidlib.c:94.
  */
-/* Called by:  zxid_mk_a7n x3, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp, zxid_wsc_prep_secmech, zxid_wsf_decor */
+/* Called by:  zxid_put_invite x2, zxid_put_psobj x2, zxid_wsc_prep_secmech, zxid_wsf_decor */
 struct zx_str* zxid_date_time(zxid_conf* cf, time_t secs)
 {
   struct tm t;
@@ -117,18 +118,19 @@ struct zx_str* zxid_date_time(zxid_conf* cf, time_t secs)
 #endif
 }
 
-struct zx_attr_s* zxid_date_time_attr(zxid_conf* cf, int tok, time_t secs)
+/* Called by:  zxid_mk_a7n x3, zxid_mk_an_stmt, zxid_mk_art_deref, zxid_mk_authn_req, zxid_mk_az, zxid_mk_az_cd1, zxid_mk_logout, zxid_mk_logout_resp, zxid_mk_mni, zxid_mk_mni_resp, zxid_mk_saml_resp, zxid_ps_addent_invite x2 */
+struct zx_attr_s* zxid_date_time_attr(zxid_conf* cf, struct zx_elem_s* father, int tok, time_t secs)
 {
   struct tm t;
   secs += cf->timeskew;
   GMTIME_R(secs, t);
 #if 0
-  /*                            "2002-10-31T21:42:14.002Z" */
-  return zx_attrf(cf->ctx, tok, "%04d-%02d-%02dT%02d:%02d:%02d.002Z",
+  /*                                    "2002-10-31T21:42:14.002Z" */
+  return zx_attrf(cf->ctx, father, tok, "%04d-%02d-%02dT%02d:%02d:%02d.002Z",
 		  t.tm_year+1900, t.tm_mon+1, t.tm_mday, t.tm_hour, t.tm_min, t.tm_sec);
 #else
-  /*                            "2002-10-31T21:42:14Z" */
-  return zx_attrf(cf->ctx, tok, "%04d-%02d-%02dT%02d:%02d:%02dZ",
+  /*                                    "2002-10-31T21:42:14Z" */
+  return zx_attrf(cf->ctx, father, tok, "%04d-%02d-%02dT%02d:%02d:%02dZ",
 		  t.tm_year+1900, t.tm_mon+1, t.tm_mday, t.tm_hour, t.tm_min, t.tm_sec);
 #endif
 }
@@ -152,7 +154,7 @@ struct zx_root_s* zxid_soap_call_envelope(zxid_conf* cf, struct zx_str* url, str
 {
   struct zx_root_s* r;
   struct zx_str* ss;
-  ss = zx_EASY_ENC_SO_e_Envelope(cf->ctx, env);
+  ss = zx_EASY_ENC_elem(cf->ctx, &env->gg);
   DD("ss(%.*s) len=%d", ss->len, ss->s, ss->len);
   r = zxid_soap_call_raw(cf, url, ss, ret_enve);
   zx_str_free(cf->ctx, ss);
@@ -171,7 +173,7 @@ struct zx_root_s* zxid_soap_call_envelope(zxid_conf* cf, struct zx_str* url, str
  * body::   XML data structure representing the SOAP body
  * return:: XML data structure representing the response  */
 
-/* Called by:  zxid_pep_az_soap, zxid_soap_call_body */
+/* Called by:  zxid_az_soap, zxid_soap_call_body */
 struct zx_root_s* zxid_soap_call_hdr_body(zxid_conf* cf, struct zx_str* url, struct zx_e_Header_s* hdr, struct zx_e_Body_s* body)
 {
   struct zx_root_s* r;
@@ -179,7 +181,10 @@ struct zx_root_s* zxid_soap_call_hdr_body(zxid_conf* cf, struct zx_str* url, str
   struct zx_e_Envelope_s* env = zx_NEW_e_Envelope(cf->ctx,0);
   env->Header = hdr;
   env->Body = body;
-  ss = zx_EASY_ENC_SO_e_Envelope(cf->ctx, env);
+  zx_add_kid(&env->gg, &body->gg);
+  if (hdr)
+    zx_add_kid(&env->gg, &hdr->gg);
+  ss = zx_EASY_ENC_elem(cf->ctx, &env->gg);
   r = zxid_soap_call_raw(cf, url, ss, 0);
   zx_str_free(cf->ctx, ss);
   return r;
@@ -211,16 +216,16 @@ struct zx_root_s* zxid_soap_call_body(zxid_conf* cf, struct zx_str* url, struct
  * body::   XML data structure representing the request
  * return:: 0 if fail, ZXID_REDIR_OK if success. */
 
-/* Called by:  zxid_idp_soap_dispatch x2, zxid_sp_soap_dispatch x6 */
+/* Called by:  zxid_idp_soap_dispatch x2, zxid_sp_soap_dispatch x7 */
 int zxid_soap_cgi_resp_body(zxid_conf* cf, struct zx_e_Body_s* body, struct zx_str* entid)
 {
   struct zx_e_Envelope_s* env = zx_NEW_e_Envelope(cf->ctx,0);
   struct zx_str* ss;
   struct zx_str* logpath;
-  
-  env->Header = zx_NEW_e_Header(cf->ctx, &env->gg);
   env->Body = body;
-  ss = zx_EASY_ENC_SO_e_Envelope(cf->ctx, env);
+  zx_add_kid(&env->gg, &body->gg);
+  env->Header = zx_NEW_e_Header(cf->ctx, &env->gg);
+  ss = zx_EASY_ENC_elem(cf->ctx, &env->gg);
 
   if (cf->log_issue_msg) {
     logpath = zxlog_path(cf, entid, ss, ZXLOG_ISSUE_DIR, ZXLOG_WIR_KIND, 1);
@@ -603,7 +608,7 @@ struct zx_str* zxid_saml2_resp_redir(zxid_conf* cf, struct zx_str* loc, struct z
  * what::   Explanatory string used in error and log messages
  * return:: 1 of SAML message is OK, 0 if message is not OK. */
 
-/* Called by:  zxid_idp_dispatch x2, zxid_idp_soap_dispatch, zxid_pep_az_soap, zxid_sp_dispatch x3, zxid_sp_mni_soap, zxid_sp_slo_soap, zxid_sp_soap_dispatch x3 */
+/* Called by:  zxid_az_soap, zxid_idp_dispatch x2, zxid_idp_soap_dispatch, zxid_sp_dispatch x3, zxid_sp_mni_soap, zxid_sp_slo_soap, zxid_sp_soap_dispatch x3 */
 int zxid_saml_ok(zxid_conf* cf, zxid_cgi* cgi, struct zx_sp_Status_s* st, char* what)
 {
   struct zx_str* ss;
@@ -655,7 +660,7 @@ int zxid_saml_ok(zxid_conf* cf, zxid_cgi* cgi, struct zx_sp_Status_s* st, char*
  *     structure is decrypted and its contents returned as the Name ID
  * return:: XML data structure corresponding to (possibly decrypted) Name ID */
 
-/* Called by:  test_ibm_cert_problem, test_ibm_cert_problem_enc_dec, zxid_di_query, zxid_idp_slo_do, zxid_mni_do, zxid_sp_slo_do, zxid_sp_sso_finalize, zxid_wsf_validate_a7n */
+/* Called by:  test_ibm_cert_problem, test_ibm_cert_problem_enc_dec, zxid_idp_map_nid2uid, zxid_idp_slo_do, zxid_mni_do, zxid_nidmap_do, zxid_sp_slo_do, zxid_sp_sso_finalize, zxid_wsf_validate_a7n */
 zxid_nid* zxid_decrypt_nameid(zxid_conf* cf, zxid_nid* nid, struct zx_sa_EncryptedID_s* encid)
 {
   struct zx_str* ss;
@@ -734,7 +739,7 @@ struct zx_str* zxid_decrypt_newnym(zxid_conf* cf, struct zx_str* newnym, struct
  * manually and then call zxsig_validate() with correctly populate refs array.
  */
 
-/* Called by:  zxid_idp_slo_do, zxid_mni_do, zxid_sp_dig_sso_a7n, zxid_sp_slo_do, zxid_xacml_az_cd1_do, zxid_xacml_az_do */
+/* Called by:  sig_validate, zxid_idp_slo_do, zxid_mni_do, zxid_sp_dig_sso_a7n, zxid_sp_slo_do, zxid_xacml_az_cd1_do, zxid_xacml_az_do */
 int zxid_chk_sig(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zx_elem_s* elem, struct zx_ds_Signature_s* sig, struct zx_sa_Issuer_s* issue_ent, struct zx_ns_s* pop_seen, const char* lk)
 {
   struct zx_str* issuer = 0;
@@ -797,7 +802,7 @@ erro:
 
 /*() Transform content according to map. The returned zx_str will be nul terminated. */
 
-/* Called by:  pool2apache x2, zxid_add_at_values, zxid_pep_az_soap x2, zxid_pool_to_json x2, zxid_pool_to_ldif x2, zxid_pool_to_qs x2 */
+/* Called by:  pool2apache x2, zxid_add_at_values, zxid_pepmap_extract x2, zxid_pool_to_json x2, zxid_pool_to_ldif x2, zxid_pool_to_qs x2 */
 struct zx_str* zxid_map_val(zxid_conf* cf, struct zxid_map* map, struct zx_str* val)
 {
   struct zx_str* ss = val;
@@ -905,6 +910,7 @@ nobody:
 
 /*() Get symmetric key, generating it if necessary. */
 
+/* Called by:  zxid_psobj_key_setup, zxlog_write_line */
 char* zx_get_symkey(zxid_conf* cf, const char* keyname, char* symkey)
 {
   char buf[1024];
