@@ -24,9 +24,6 @@
 #include "c/zx-data.h"
 #include "c/zx-e-data.h"
 
-#define zx_e_actor_ATTR (zx_e_NS|zx_actor_ATTR)
-#define zx_e_mustUnderstand_ATTR (zx_e_NS|zx_mustUnderstand_ATTR)
-
 #if 0
 #define XS_STRING "xs:string"
 #else
@@ -50,6 +47,7 @@
  * env::  Value-result parameter. Linked list of environment attributes.
  */
 
+/* Called by:  zxid_pep_az_base_soap_pepmap, zxid_pep_az_soap_pepmap */
 static void zxid_pepmap_extract(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zxid_map* pepmap, struct zx_xac_Attribute_s** subj, struct zx_xac_Attribute_s** rsrc, struct zx_xac_Attribute_s** act, struct zx_xac_Attribute_s** env)
 {
   struct zxid_map* map;
@@ -159,6 +157,7 @@ static void zxid_pepmap_extract(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, str
  * returns:: SAML Response as data structure or null upon error.
  */
 
+/* Called by:  zxid_pep_az_base_soap_pepmap, zxid_pep_az_soap_pepmap */
 static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const char* pdp_url, struct zx_xac_Attribute_s* subj, struct zx_xac_Attribute_s* rsrc, struct zx_xac_Attribute_s* act, struct zx_xac_Attribute_s* env)
 {
   X509* sign_cert;
@@ -179,58 +178,64 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
 
   //zx_add_content(cf->ctx, &hdr->Action->gg, zx_dup_str(cf->ctx, "SAML2XACMLAuthzRequest"));
   //zx_add_content(cf->ctx, &hdr->Action->gg, zx_dup_str(cf->ctx, "http://ws.apache.org/axis2/TestPolicyPortType/authRequestRequest"));
-  hdr->Action->actor = zx_ref_str(cf->ctx, SOAP_ACTOR_NEXT);
   hdr->Action->mustUnderstand = zx_ref_str(cf->ctx, ZXID_TRUE);
+  hdr->Action->actor = zx_ref_str(cf->ctx, SOAP_ACTOR_NEXT);
 #endif
 
   /* Add our own token so PDP can do whatever PEP can (they are considered to be
    * part of the same entity). This is TAS3 specific hack. */
 
   sec = hdr->Security = zx_NEW_wsse_Security(cf->ctx, &hdr->gg);
-  sec->actor = zx_ref_attr(cf->ctx, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
-  sec->mustUnderstand = zx_ref_attr(cf->ctx, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
+  sec->actor = zx_ref_attr(cf->ctx, &sec->gg, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
+  sec->mustUnderstand = zx_ref_attr(cf->ctx, &sec->gg, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
   sec->Timestamp = zx_NEW_wsu_Timestamp(cf->ctx, &sec->gg);
   sec->Timestamp->Created = zx_NEW_wsu_Created(cf->ctx, &sec->Timestamp->gg);
   sec->Assertion = ses->tgta7n;
+  zx_reverse_elem_lists(&sec->gg);
   D("tgta7n=%p", ses->tgta7n);
 
   /* Prepare request according to the version */
 
   body = zx_NEW_e_Body(cf->ctx,0);
   if (!strcmp(cf->xasp_vers, "xac-soap")) {
-    body->xac_Request = zxid_mk_xac_az(cf, subj, rsrc, act, env);
+    ZX_ADD_KID(body, xac_Request, zxid_mk_xac_az(cf, &body->gg, subj, rsrc, act, env));
 #if 0
     /* *** xac:Response does not have signature field */
     if (cf->sso_soap_sign) {
       ZERO(&refs, sizeof(refs));
       refs.id = body->xac_Request->ID;
       refs.canon = zx_EASY_ENC_SO_xac_Request(cf->ctx, body->xac_Request);
-      if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az xac-soap"))
-	body->xac_Request->Signature
-	  = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
+      if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az xac-soap")) {
+	body->xac_Request->Signature = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
+	zx_add_kid(&body->xac_Request->gg, &body->xac_Request->Signature->gg);
+      }
       zx_str_free(cf->ctx, refs.canon);
     }
 #endif
   } else if (!strcmp(cf->xasp_vers, "2.0-cd1")) {
-    body->xaspcd1_XACMLAuthzDecisionQuery = zxid_mk_az_cd1(cf, subj, rsrc, act, env);
+    ZX_ADD_KID(body, xaspcd1_XACMLAuthzDecisionQuery, zxid_mk_az_cd1(cf, subj, rsrc, act, env));
     if (cf->sso_soap_sign) {
       ZERO(&refs, sizeof(refs));
       refs.id = &body->xaspcd1_XACMLAuthzDecisionQuery->ID->g;
-      refs.canon = zx_EASY_ENC_SO_xaspcd1_XACMLAuthzDecisionQuery(cf->ctx, body->xaspcd1_XACMLAuthzDecisionQuery);
-      if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az cd1"))
+      refs.canon = zx_EASY_ENC_elem(cf->ctx, &body->xaspcd1_XACMLAuthzDecisionQuery->gg);
+      if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az cd1")) {
 	body->xaspcd1_XACMLAuthzDecisionQuery->Signature
 	  = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
+	zx_add_kid_after_sa_Issuer(&body->xaspcd1_XACMLAuthzDecisionQuery->gg, &body->xaspcd1_XACMLAuthzDecisionQuery->Signature->gg);
+      }
       zx_str_free(cf->ctx, refs.canon);
     }
   } else {
-    body->XACMLAuthzDecisionQuery = zxid_mk_az(cf, subj, rsrc, act, env);
+    ZX_ADD_KID(body, XACMLAuthzDecisionQuery, zxid_mk_az(cf, subj, rsrc, act, env));
     if (cf->sso_soap_sign) {
       ZERO(&refs, sizeof(refs));
       refs.id = &body->XACMLAuthzDecisionQuery->ID->g;
-      refs.canon = zx_EASY_ENC_SO_xasp_XACMLAuthzDecisionQuery(cf->ctx, body->XACMLAuthzDecisionQuery);
-      if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az"))
+      refs.canon = zx_EASY_ENC_elem(cf->ctx, &body->XACMLAuthzDecisionQuery->gg);
+      if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az")) {
 	body->XACMLAuthzDecisionQuery->Signature
 	  = zxsig_sign(cf->ctx, 1, &refs, sign_cert, sign_pkey);
+	zx_add_kid_after_sa_Issuer(&body->XACMLAuthzDecisionQuery->gg, &body->XACMLAuthzDecisionQuery->Signature->gg);
+      }
       zx_str_free(cf->ctx, refs.canon);
     }
   }
@@ -290,13 +295,13 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
  * For simpler API, see zxid_az() family of functions.
  */
 
+/* Called by:  zxid_call_epr, zxid_pep_az_soap, zxid_simple_ab_pep, zxid_simple_ses_active_cf, zxid_wsc_prepare_call, zxid_wsc_validate_resp_env, zxid_wsp_decorate, zxid_wsp_validate */
 char* zxid_pep_az_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const char* pdp_url, struct zxid_map* pepmap)
 {
   struct zx_xac_Attribute_s* subj = 0;
   struct zx_xac_Attribute_s* rsrc = 0;
   struct zx_xac_Attribute_s* act = 0;
   struct zx_xac_Attribute_s* env = 0;
-  char* res;
   struct zx_str* ss;
   struct zx_sp_Response_s* resp;
   struct zx_sa_Statement_s* stmt;
@@ -314,50 +319,53 @@ char* zxid_pep_az_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const
 
   zxid_pepmap_extract(cf, cgi, ses, pepmap, &subj, &rsrc, &act, &env);
   resp = zxid_az_soap(cf, cgi, ses, pdp_url, subj, rsrc, act, env);
-  if (!resp)
+  if (!resp || !resp->Assertion) {
+    ERR("DENY due to malformed authorization response from PDP. Either no response or response lacjing assertion. %p", resp);
     return 0;
-
+  }
+  
   az_stmt = resp->Assertion->XACMLAuthzDecisionStatement;
   if (az_stmt && az_stmt->Response && az_stmt->Response->Result) {
     decision = az_stmt->Response->Result->Decision;
     if (ZX_CONTENT_EQ_CONST(decision, "Permit")) {
-      ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &az_stmt->Response->gg);
+      ss = zx_EASY_ENC_elem(cf->ctx, &az_stmt->Response->gg);
       if (!ss || !ss->len)
 	return 0;
-      res = ss->s;
       ZX_FREE(cf->ctx, ss);
-      D("Permit azstmt(%s)", res);
-      return res;
+      DD("Permit azstmt(%s)", ss->s);
+      INFO("PERMIT found in azstmt len=%d", ss->len);
+      return ss->s;
     }
   }
   az_stmt_cd1 = resp->Assertion->xasacd1_XACMLAuthzDecisionStatement;
   if (az_stmt_cd1 && az_stmt_cd1->Response && az_stmt_cd1->Response->Result) {
     decision = az_stmt_cd1->Response->Result->Decision;
     if (ZX_CONTENT_EQ_CONST(decision, "Permit")) {
-      ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &az_stmt_cd1->Response->gg);
+      ss = zx_EASY_ENC_elem(cf->ctx, &az_stmt_cd1->Response->gg);
       if (!ss || !ss->len)
 	return 0;
-      res = ss->s;
       ZX_FREE(cf->ctx, ss);
-      D("Permit cd1(%s)", res);
-      return res;
+      DD("Permit cd1(%s)", ss->s);
+      INFO("PERMIT found in azstmt_cd1 len=%d", ss->len);
+      return ss->s;
     }
   }
   stmt = resp->Assertion->Statement;
   if (stmt && stmt->Response && stmt->Response->Result) {  /* Response here is xac:Response */
     decision = stmt->Response->Result->Decision;
     if (ZX_CONTENT_EQ_CONST(decision, "Permit")) {
-      ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &stmt->Response->gg);
+      ss = zx_EASY_ENC_elem(cf->ctx, &stmt->Response->gg);
       if (!ss || !ss->len)
 	return 0;
-      res = ss->s;
       ZX_FREE(cf->ctx, ss);
-      D("Permit stmt(%s)", res);
-      return res;
+      D("Permit stmt(%s)", ss->s);
+      INFO("PERMIT found in stmt len=%d", ss->len);
+      return ss->s;
     }
   }
   /*if (resp->Assertion->AuthzDecisionStatement) {  }*/
   D("Deny or error or no xac:Response in reply %d",0);
+  INFO("DENY or error or no xac:Response from PDP %p %p %p", az_stmt, az_stmt_cd1, stmt);
   return 0;
 }
 
@@ -388,6 +396,7 @@ char* zxid_pep_az_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const
  * For simpler API, see zxid_az_base() family of functions.
  */
 
+/* Called by:  zxid_pep_az_base_soap */
 char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const char* pdp_url, struct zxid_map* pepmap)
 {
   struct zx_xac_Attribute_s* subj = 0;
@@ -416,7 +425,7 @@ char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses,
 
   az_stmt = resp->Assertion->XACMLAuthzDecisionStatement;
   if (az_stmt && az_stmt->Response) {
-    ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &az_stmt->Response->gg);
+    ss = zx_EASY_ENC_elem(cf->ctx, &az_stmt->Response->gg);
     if (!ss || !ss->len)
       return 0;
     res = ss->s;
@@ -426,7 +435,7 @@ char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses,
   }
   az_stmt_cd1 = resp->Assertion->xasacd1_XACMLAuthzDecisionStatement;
   if (az_stmt_cd1 && az_stmt_cd1->Response) {
-    ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &az_stmt_cd1->Response->gg);
+    ss = zx_EASY_ENC_elem(cf->ctx, &az_stmt_cd1->Response->gg);
     if (!ss || !ss->len)
       return 0;
     res = ss->s;
@@ -436,7 +445,7 @@ char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses,
   }
   stmt = resp->Assertion->Statement;
   if (stmt && stmt->Response) {  /* Response here is xac:Response */
-    ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &stmt->Response->gg);
+    ss = zx_EASY_ENC_elem(cf->ctx, &stmt->Response->gg);
     if (!ss || !ss->len)
       return 0;
     res = ss->s;
@@ -452,11 +461,12 @@ char* zxid_pep_az_base_soap_pepmap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses,
 /*() Call Policy Decision Point (PDP) to obtain an authorization decision.
  * Uses default PEPMAP to call zxid_pep_az_soap_pepmap(). */
 
-/* Called by:  zxid_az_cf_ses, zxid_call x2, zxid_simple_ab_pep, zxid_simple_ses_active_cf, zxid_wsc_prepare_call, zxid_wsc_valid_resp */
+/* Called by:  zxid_az_cf_ses */
 char* zxid_pep_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const char* pdp_url) {
   return zxid_pep_az_soap_pepmap(cf, cgi, ses, pdp_url, cf->pepmap);
 }
 
+/* Called by:  zxid_az_base_cf_ses */
 char* zxid_pep_az_base_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, const char* pdp_url) {
   return zxid_pep_az_base_soap_pepmap(cf, cgi, ses, pdp_url, cf->pepmap);
 }
@@ -496,6 +506,7 @@ char* zxid_az_cf_ses(zxid_conf* cf, const char* qs, zxid_ses* ses)
   return ret;
 }
 
+/* Called by:  zxid_az_base_cf */
 char* zxid_az_base_cf_ses(zxid_conf* cf, const char* qs, zxid_ses* ses)
 {
   zxid_cgi cgi;
@@ -538,6 +549,7 @@ char* zxid_az_cf(zxid_conf* cf, const char* qs, const char* sid)
   return zxid_az_cf_ses(cf, qs, &ses);
 }
 
+/* Called by:  zxid_az_base */
 char* zxid_az_base_cf(zxid_conf* cf, const char* qs, const char* sid)
 {
   zxid_ses ses;
@@ -562,6 +574,7 @@ char* zxid_az(const char* conf, const char* qs, const char* sid)
   return zxid_az_cf(&cf, qs, sid);
 }
 
+/* Called by: */
 char* zxid_az_base(const char* conf, const char* qs, const char* sid)
 {
   struct zx_ctx ctx;
