@@ -56,8 +56,8 @@ int zxid_pick_sso_profile(zxid_conf* cf, zxid_cgi* cgi, zxid_entity* idp_meta)
 }
 
 /*() Map name id format form field to SAML specified URN string. */
-/* Called by:  zxid_mk_authn_req */
-char* zxid_saml2_map_nid_fmt(char* f)
+/* Called by:  zxid_map_identity_token, zxid_mk_authn_req */
+const char* zxid_saml2_map_nid_fmt(const char* f)
 {
   switch (f[0]) {
   case 'n' /*'none'*/:   return "";
@@ -75,7 +75,7 @@ char* zxid_saml2_map_nid_fmt(char* f)
 
 /*() Map protocol binding form field to SAML specified URN string. */
 /* Called by: */
-char* zxid_saml2_map_protocol_binding(char* b)
+const char* zxid_saml2_map_protocol_binding(const char* b)
 {
   switch (b[0]) {
   case 'r' /*'redir'*/: return SAML2_REDIR;
@@ -179,7 +179,7 @@ struct zx_str* zxid_start_sso_url(zxid_conf* cf, zxid_cgi* cgi)
     }
     ar = zxid_mk_authn_req(cf, cgi);
     ar->Destination = sso_svc->Location;
-    ars = zx_EASY_ENC_SO_sp_AuthnRequest(cf->ctx, ar);
+    ars = zx_EASY_ENC_elem(cf->ctx, &ar->gg);
     D("AuthnReq(%.*s)", ars->len, ars->s);
     break;
   default:
@@ -243,6 +243,14 @@ int zxid_sp_deref_art(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
   char* p;
   char buf[64];
   D_INDENT("deref: ");
+
+  if (!cgi || !cgi->saml_art || !*cgi->saml_art) {
+    ERR("SAMLart missing or empty string. %p %p", cgi, cgi?cgi->saml_art:0);
+    zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "C", "ERR", cgi?cgi->saml_art:0, "Artifact missing");
+    D_DEDENT("deref: ");
+    return 0;
+  }
+  
   len = strlen(cgi->saml_art);
   if (cf->log_level > 0)
     zxlog(cf, 0, 0, 0, 0, 0, 0, ZX_GET_CONTENT(ses->nameid), "N", "W", "ART", cgi->saml_art, 0);
@@ -320,7 +328,7 @@ int zxid_sp_deref_art(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses)
 
 /*() Map ZXSIG constant to letter for log and string message. */
 
-/* Called by:  zxid_chk_sig, zxid_decode_redir_or_post, zxid_sp_sso_finalize, zxid_wsf_validate_a7n, zxid_wsp_validate */
+/* Called by:  zxid_chk_sig, zxid_decode_redir_or_post, zxid_sp_sso_finalize, zxid_wsc_validate_resp_env, zxid_wsf_validate_a7n, zxid_wsp_validate */
 void zxid_sigres_map(int sigres, char** sigval, char** sigmsg)
 {
   switch (sigres) {
@@ -392,7 +400,7 @@ void zxid_sigres_map(int sigres, char** sigval, char** sigmsg)
  * cgi::     Optional CGI object. If non-NULL, sigval and sigmsg will be set.
  * ses::     Optional session object. If non-NULL, then sigres code will be set.
  * a7n::     Assertion whose conditions are checked.
- * myentid:: Entity ID used for checking audience restriction. Typically from zxid_my_entity_id(cf)
+ * myentid:: Entity ID used for checking audience restriction. Typically from zxid_my_ent_id(cf)
  * ourts::   Timestamp for validating NotOnOrAfter and NotBefore.
  * err::     Result argument: Error letter (as may appear in audit log entry). The returned
  *     string will be a constant and MUST NOT be freed by the caller.
@@ -520,7 +528,7 @@ struct zx_str unknown_str = {0,0,1,"??"};  /* Static string used as dummy value.
  * a7n:: Single Sign-On assertion
  * return:: 0 for failure, otherwise some success code such as ZXID_SSO_OK */
 
-/* Called by:  main, zxid_sp_dig_sso_a7n */
+/* Called by:  main, sig_validate, zxid_sp_dig_sso_a7n */
 int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n* a7n, struct zx_ns_s* pop_seen)
 {
   char* err = "S"; /* See: RES in zxid-log.pd, section "ZXID Log Format" */
@@ -629,7 +637,7 @@ int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n*
     goto erro;
   }
   
-  if (zxid_validate_cond(cf, cgi, ses, a7n, zxid_my_entity_id(cf), &ourts, &err))
+  if (zxid_validate_cond(cf, cgi, ses, a7n, zxid_my_ent_id(cf), &ourts, &err))
     goto erro;
   
   if (cf->log_rely_a7n) {
@@ -637,7 +645,7 @@ int zxid_sp_sso_finalize(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, zxid_a7n*
     logpath = zxlog_path(cf, issuer, &a7n->ID->g, ZXLOG_RELY_DIR, ZXLOG_A7N_KIND, 1);
     if (logpath) {
       ses->sso_a7n_path = ses->tgt_a7n_path = zx_str_to_c(cf->ctx, logpath);
-      ss = zx_EASY_ENC_WO_any_elem(cf->ctx, &a7n->gg);
+      ss = zx_EASY_ENC_elem(cf->ctx, &a7n->gg);
       if (zxlog_dup_check(cf, logpath, "SSO assertion")) {
 	if (cf->dup_a7n_fatal) {
 	  err = "C";
@@ -779,8 +787,8 @@ int zxid_as_call_ses(zxid_conf* cf, zxid_entity* idp_meta, zxid_cgi* cgi, zxid_s
   
   body = zx_NEW_e_Body(cf->ctx,0);
   body->SASLRequest = zx_NEW_as_SASLRequest(cf->ctx, &body->gg);
-  body->SASLRequest->mechanism = zx_dup_attr(cf->ctx, zx_mechanism_ATTR, "PLAIN");
-  body->SASLRequest->Data = zx_ref_len_simple_elem(cf->ctx, &body->SASLRequest->gg, zx_as_Data_ELEM, p-b64, b64);
+  body->SASLRequest->mechanism = zx_dup_attr(cf->ctx, &body->SASLRequest->gg, zx_mechanism_ATTR, "PLAIN");
+  body->SASLRequest->Data = zx_ref_len_elem(cf->ctx, &body->SASLRequest->gg, zx_as_Data_ELEM, p-b64, b64);
   r = zxid_soap_call_body(cf, &ar_svc->Location->g, body);
   /* *** free the body */
   
