@@ -25,9 +25,6 @@
 #include "c/zx-ns.h"
 #include "c/zx-data.h"
 
-#define zx_e_actor_ATTR (zx_e_NS|zx_actor_ATTR)
-#define zx_e_mustUnderstand_ATTR (zx_e_NS|zx_mustUnderstand_ATTR)
-
 #define XS_STRING "http://www.w3.org/2001/XMLSchema#string"
 #define BOOL_STR_TEST(x) ((x) && (x) != '0')
 
@@ -115,7 +112,7 @@ int zxid_map_sec_mech(zxid_epr* epr)
  * of all apparently signed message parts.
  * See also: zxid_add_header_refs() and zxsig_sign() or zxid_chk_sig() + zxsig_validate() */
 
-/* Called by:  zxid_wsf_sign */
+/* Called by:  wsse_sec_validate, zxid_wsc_validate_resp_env, zxid_wsp_validate */
 int zxid_hunt_sig_parts(zxid_conf* cf, int n_refs, struct zxsig_ref* refs, struct zx_ds_Reference_s* sref, struct zx_e_Header_s* hdr, struct zx_e_Body_s* bdy)
 {
   for (; sref && sref->gg.g.tok == zx_ds_Reference_ELEM
@@ -356,6 +353,20 @@ int zxid_hunt_sig_parts(zxid_conf* cf, int n_refs, struct zxsig_ref* refs, struc
   }
   return n_refs;
 }
+
+#define ZXID_ADD_WSU_ID(H,idval) MB if (!H->Id) \
+  H->Id = zx_ord_ins_at(&H->gg, zx_ref_attr(cf->ctx, 0, zx_wsu_Id_ATTR, idval)); \
+  refs[n_refs].id = &H->Id->g; \
+  refs[n_refs].canon = zx_EASY_ENC_elem(cf->ctx, &H->gg); \
+  ++n_refs; ME
+
+#define ZXID_ADD_ID(H,idval) MB if (!H->id) \
+  H->id = zx_ord_ins_at(&H->gg, zx_ref_attr(cf->ctx, 0, zx_id_ATTR, idval)); \
+  refs[n_refs].id = &H->id->g; \
+  refs[n_refs].canon = zx_EASY_ENC_elem(cf->ctx, &H->gg); \
+  ++n_refs; ME
+
+
 /*() For purposes of signing, add references and canon forms of all known SOAP headers.
  * N.B. This function only works for preparing for signing.
  * See also: zxsig_sign(), zxid_hunt_sig_parts() or zxid_chk_sig() + zxsig_validate() */
@@ -365,244 +376,74 @@ int zxid_add_header_refs(zxid_conf* cf, int n_refs, struct zxsig_ref* refs, stru
 {
   /* Addressing and Security Headers */
 
-  if (hdr->Framework) {
-    if (!hdr->Framework->Id)
-      hdr->Framework->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "FWK");
-    refs[n_refs].id = &hdr->Framework->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_sbf_Framework(cf->ctx, hdr->Framework);
-    ++n_refs;
-  }
-  
-  if (hdr->Security && hdr->Security->Timestamp) {
-    if (!hdr->Security->Timestamp->Id)
-      hdr->Security->Timestamp->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "TS");
-    refs[n_refs].id = &hdr->Security->Timestamp->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_wsu_Timestamp(cf->ctx, hdr->Security->Timestamp);
-    ++n_refs;
-  }
-
-  if (hdr->MessageID) {
-    if (!hdr->MessageID->Id)
-      hdr->MessageID->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "MID");
-    refs[n_refs].id = &hdr->MessageID->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_a_MessageID(cf->ctx, hdr->MessageID);
-    ++n_refs;
-  }
-  
-  if (hdr->RelatesTo) {
-    if (!hdr->RelatesTo->Id)
-      hdr->RelatesTo->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "REL");
-    refs[n_refs].id = &hdr->RelatesTo->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_a_RelatesTo(cf->ctx, hdr->RelatesTo);
-    ++n_refs;
-  }
-  
-  if (hdr->Action) {
-    if (!hdr->Action->Id)
-      hdr->Action->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "ACT");
-    refs[n_refs].id = &hdr->Action->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_a_Action(cf->ctx, hdr->Action);
-    ++n_refs;
-  }
-
-  if (hdr->To) {
-    if (!hdr->To->Id)
-      hdr->To->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "TO");
-    refs[n_refs].id = &hdr->To->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_a_To(cf->ctx, hdr->To);
-    ++n_refs;
-  }
-
-  if (hdr->ReplyTo) {
-    if (!hdr->ReplyTo->Id)
-      hdr->ReplyTo->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "REP");
-    refs[n_refs].id = &hdr->ReplyTo->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_a_ReplyTo(cf->ctx, hdr->ReplyTo);
-    ++n_refs;
-  }
-  
-  if (hdr->From) {
-    if (!hdr->From->Id)
-      hdr->From->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "FRM");
-    refs[n_refs].id = &hdr->From->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_a_From(cf->ctx, hdr->From);
-    ++n_refs;
-  }
-  
-  if (hdr->Sender) {
-    if (!hdr->Sender->Id)
-      hdr->Sender->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "PRV");
-    refs[n_refs].id = &hdr->Sender->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_Sender(cf->ctx, hdr->Sender);
-    ++n_refs;
-  }
-  
-  if (hdr->FaultTo) {
-    if (!hdr->FaultTo->Id)
-      hdr->FaultTo->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "FLT");
-    refs[n_refs].id = &hdr->FaultTo->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_a_FaultTo(cf->ctx, hdr->FaultTo);
-    ++n_refs;
-  }
-  
-  if (hdr->ReferenceParameters) {
-    if (!hdr->ReferenceParameters->Id)
-      hdr->ReferenceParameters->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "PAR");
-    refs[n_refs].id = &hdr->ReferenceParameters->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_a_ReferenceParameters(cf->ctx, hdr->ReferenceParameters);
-    ++n_refs;
-  }
+  if (hdr->Framework)
+    ZXID_ADD_WSU_ID(hdr->Framework,"FWK");
+  if (hdr->Security && hdr->Security->Timestamp)
+    ZXID_ADD_WSU_ID(hdr->Security->Timestamp,"TS");
+  if (hdr->MessageID)
+    ZXID_ADD_WSU_ID(hdr->MessageID,"MID");
+  if (hdr->RelatesTo)
+    ZXID_ADD_WSU_ID(hdr->RelatesTo,"REL");
+  if (hdr->Action)
+    ZXID_ADD_WSU_ID(hdr->Action,"ACT");
+  if (hdr->To)
+    ZXID_ADD_WSU_ID(hdr->To,"TO");
+  if (hdr->ReplyTo)
+    ZXID_ADD_WSU_ID(hdr->ReplyTo,"REP");
+  if (hdr->From)
+    ZXID_ADD_WSU_ID(hdr->From,"FRM");
+  if (hdr->Sender)
+    ZXID_ADD_WSU_ID(hdr->Sender,"PRV");
+  if (hdr->FaultTo)
+    ZXID_ADD_WSU_ID(hdr->FaultTo,"FLT");
+  if (hdr->ReferenceParameters)
+    ZXID_ADD_WSU_ID(hdr->ReferenceParameters,"PAR");
   
   /* ID-WSF headers */
   
-  if (hdr->TargetIdentity) {
-    if (!hdr->TargetIdentity->Id)
-      hdr->TargetIdentity->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "TRG");
-    refs[n_refs].id = &hdr->TargetIdentity->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_TargetIdentity(cf->ctx, hdr->TargetIdentity);
-    ++n_refs;
-  }
-  
-  if (hdr->UsageDirective) {
-    if (!hdr->UsageDirective->Id)
-      hdr->UsageDirective->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "UD");
-    refs[n_refs].id = &hdr->UsageDirective->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_UsageDirective(cf->ctx, hdr->UsageDirective);
-    ++n_refs;
-  }
-  
-  if (hdr->UserInteraction) {
-    if (!hdr->UserInteraction->Id)
-      hdr->UserInteraction->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "UI");
-    refs[n_refs].id = &hdr->UserInteraction->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_UserInteraction(cf->ctx, hdr->UserInteraction);
-    ++n_refs;
-  }
-  
-  if (hdr->ProcessingContext) {
-    if (!hdr->ProcessingContext->Id)
-      hdr->ProcessingContext->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "PC");
-    refs[n_refs].id = &hdr->ProcessingContext->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_ProcessingContext(cf->ctx, hdr->ProcessingContext);
-    ++n_refs;
-  }
-  
-  if (hdr->EndpointUpdate) {
-    if (!hdr->EndpointUpdate->Id)
-      hdr->EndpointUpdate->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "EP");
-    refs[n_refs].id = &hdr->EndpointUpdate->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_EndpointUpdate(cf->ctx, hdr->EndpointUpdate);
-    ++n_refs;
-  }
-  
-  if (hdr->Timeout) {
-    if (!hdr->Timeout->Id)
-      hdr->Timeout->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "TI");
-    refs[n_refs].id = &hdr->Timeout->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_Timeout(cf->ctx, hdr->Timeout);
-    ++n_refs;
-  }
-  
-  if (hdr->Consent) {
-    if (!hdr->Consent->Id)
-      hdr->Consent->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "CON");
-    refs[n_refs].id = &hdr->Consent->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_Consent(cf->ctx, hdr->Consent);
-    ++n_refs;
-  }
-  
-  if (hdr->ApplicationEPR) {
-    if (!hdr->ApplicationEPR->Id)
-      hdr->ApplicationEPR->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "AEP");
-    refs[n_refs].id = &hdr->ApplicationEPR->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_ApplicationEPR(cf->ctx, hdr->ApplicationEPR);
-    ++n_refs;
-  }
-  
-  if (hdr->RedirectRequest) {
-    if (!hdr->RedirectRequest->Id)
-      hdr->RedirectRequest->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "RR");
-    refs[n_refs].id = &hdr->RedirectRequest->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_RedirectRequest(cf->ctx, hdr->RedirectRequest);
-    ++n_refs;
-  }
-  
-  if (hdr->CredentialsContext) {
-    if (!hdr->CredentialsContext->Id)
-      hdr->CredentialsContext->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "CCX");
-    refs[n_refs].id = &hdr->CredentialsContext->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b_CredentialsContext(cf->ctx, hdr->CredentialsContext);
-    ++n_refs;
-  }
+  if (hdr->TargetIdentity)
+    ZXID_ADD_WSU_ID(hdr->TargetIdentity,"TRG");
+  if (hdr->UsageDirective)
+    ZXID_ADD_WSU_ID(hdr->UsageDirective,"UD");
+  if (hdr->UserInteraction)
+    ZXID_ADD_WSU_ID(hdr->UserInteraction,"UI");
+  if (hdr->ProcessingContext)
+    ZXID_ADD_WSU_ID(hdr->ProcessingContext,"PC");
+  if (hdr->EndpointUpdate)
+    ZXID_ADD_WSU_ID(hdr->EndpointUpdate,"EP");
+  if (hdr->Timeout)
+    ZXID_ADD_WSU_ID(hdr->Timeout,"TI");
+  if (hdr->Consent)
+    ZXID_ADD_WSU_ID(hdr->Consent,"CON");
+  if (hdr->ApplicationEPR)
+    ZXID_ADD_WSU_ID(hdr->ApplicationEPR,"AEP");
+  if (hdr->RedirectRequest)
+    ZXID_ADD_WSU_ID(hdr->RedirectRequest,"RR");
+  if (hdr->CredentialsContext)
+    ZXID_ADD_WSU_ID(hdr->CredentialsContext,"CCX");
   
   /* TAS3 specifics */
   
-  if (hdr->Credentials) {
-    if (!hdr->Credentials->Id)
-      hdr->Credentials->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "CRED");
-    refs[n_refs].id = &hdr->Credentials->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_tas3_Credentials(cf->ctx, hdr->Credentials);
-    ++n_refs;
-  }
-  
-  if (hdr->ESLPolicies) {
-    if (!hdr->ESLPolicies->Id)
-      hdr->ESLPolicies->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "ESL");
-    refs[n_refs].id = &hdr->ESLPolicies->Id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_tas3_ESLPolicies(cf->ctx, hdr->ESLPolicies);
-    ++n_refs;
-  }
+  if (hdr->Credentials)
+    ZXID_ADD_WSU_ID(hdr->Credentials,"CRED");
+  if (hdr->ESLPolicies)
+    ZXID_ADD_WSU_ID(hdr->ESLPolicies,"ESL");
   
   /* Old ID-WSF 1.2 Headers and App specific headers */
   
-  if (hdr->Correlation) {
-    if (!hdr->Correlation->id)
-      hdr->Correlation->id = zx_ref_attr(cf->ctx, zx_id_ATTR, "COR");
-    refs[n_refs].id = &hdr->Correlation->id->g;
-    refs[n_refs].canon
-      = zx_EASY_ENC_SO_b12_Correlation(cf->ctx, hdr->Correlation);
-    ++n_refs;
-  }
-  
-  if (hdr->Provider) {
-    if (!hdr->Provider->id)
-      hdr->Provider->id = zx_ref_attr(cf->ctx, zx_id_ATTR, "PC12");
-    refs[n_refs].id = &hdr->Provider->id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b12_Provider(cf->ctx, hdr->Provider);
-    ++n_refs;
-  }
-  
-  if (hdr->b12_ProcessingContext) {
-    if (!hdr->b12_ProcessingContext->id)
-      hdr->b12_ProcessingContext->id = zx_ref_attr(cf->ctx, zx_id_ATTR, "PC12");
-    refs[n_refs].id = &hdr->b12_ProcessingContext->id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b12_ProcessingContext(cf->ctx, hdr->b12_ProcessingContext);
-    ++n_refs;
-  }
-  
-  if (hdr->b12_Consent) {
-    if (!hdr->b12_Consent->id)
-      hdr->b12_Consent->id = zx_ref_attr(cf->ctx, zx_id_ATTR, "CON12");
-    refs[n_refs].id = &hdr->b12_Consent->id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b12_Consent(cf->ctx, hdr->b12_Consent);
-    ++n_refs;
-  }
-  
-  if (hdr->b12_UsageDirective) {
-    if (!hdr->b12_UsageDirective->id)
-      hdr->b12_UsageDirective->id = zx_ref_attr(cf->ctx, zx_id_ATTR, "UD12");
-    refs[n_refs].id = &hdr->b12_UsageDirective->id->g;
-    refs[n_refs].canon = zx_EASY_ENC_SO_b12_UsageDirective(cf->ctx, hdr->b12_UsageDirective);
-    ++n_refs;
-  }
+  if (hdr->Correlation)
+    ZXID_ADD_ID(hdr->Correlation,"COR");
+  if (hdr->Provider)
+    ZXID_ADD_ID(hdr->Provider,"PROV12");
+  if (hdr->b12_ProcessingContext)
+    ZXID_ADD_ID(hdr->b12_ProcessingContext,"PC12");
+  if (hdr->b12_Consent)
+    ZXID_ADD_ID(hdr->b12_Consent,"CON12");
+  if (hdr->b12_UsageDirective)
+    ZXID_ADD_ID(hdr->b12_UsageDirective,"UD12");
 #if 0
-  if (hdr->TransactionID) {
-    if (!hdr->TransactionID->id)
-      hdr->TransactionID->id = zx_ref_attr(cf->ctx, zx_id_ATTR, "MM7TX");
-    refs[n_refs].id = &hdr->TransactionID->id->g;  /* *** mm7:TransactionID does not have id or Id */
-    refs[n_refs].canon = zx_EASY_ENC_SO_mm7_TransactionID(cf->ctx, hdr->TransactionID);
-    ++n_refs;
-  }
+  if (hdr->TransactionID)
+    ZXID_ADD_ID(hdr->TransactionID,"MM7TX"); /* *** mm7:TransactionID does not have id or Id */
 #endif
   return n_refs;
 }
@@ -623,26 +464,18 @@ void zxid_wsf_sign(zxid_conf* cf, int sign_flags, struct zx_wsse_Security_s* sec
     if (sign_flags & ZXID_SIGN_HDR)
       n_refs = zxid_add_header_refs(cf, n_refs, refs, hdr);
     
-    if (str) {
-      if (!str->Id)
-	str->Id = zx_ref_attr(cf->ctx, zx_Id_ATTR, "STR");
-      refs[n_refs].id = &str->Id->g;
-      refs[n_refs].canon = zx_EASY_ENC_SO_wsse_SecurityTokenReference(cf->ctx, str);
-      ++n_refs;
-    }
+    if (str)
+      ZXID_ADD_WSU_ID(str,"STR");
     
-    if (bdy && (sign_flags & ZXID_SIGN_BDY)) {
-      if (!bdy->id)
-	bdy->id = zx_ref_attr(cf->ctx, zx_id_ATTR, "BDY");
-      refs[n_refs].id = &bdy->id->g;
-      refs[n_refs].canon = zx_EASY_ENC_SO_e_Body(cf->ctx, bdy);
-      ++n_refs;
-    }
+    if (bdy && (sign_flags & ZXID_SIGN_BDY))
+      ZXID_ADD_ID(bdy,"BDY");
    
     ASSERTOP(n_refs, <=, ZXID_N_WSF_SIGNED_HEADERS);
 
-    if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert wsc"))
+    if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert wsc")) {
       sec->Signature = zxsig_sign(cf->ctx, n_refs, refs, sign_cert, sign_pkey);
+      zx_add_kid(&sec->gg, &sec->Signature->gg);
+    }
   }
 }
 
@@ -650,7 +483,8 @@ void zxid_wsf_sign(zxid_conf* cf, int sign_flags, struct zx_wsse_Security_s* sec
  * The validity is controlled by configuration parameters BEFORE_SLOP and AFTER_SLOP.
  * returns 1 on success, 0 on failure. */
 
-int zxid_wsf_timestamp_check(zxid_conf* cf, zxid_ses* ses, struct zx_wsu_Timestamp_s* ts, struct timeval* ourts, struct timeval* srcts, const char* ctlpt, const char* faultactor)
+/* Called by:  zxid_wsc_valid_re_env, zxid_wsp_validate */
+int zxid_timestamp_chk(zxid_conf* cf, zxid_ses* ses, struct zx_wsu_Timestamp_s* ts, struct timeval* ourts, struct timeval* srcts, const char* ctlpt, const char* faultactor)
 {
   if (ts && ZX_SIMPLE_ELEM_CHK(ts->Created)) {
     srcts->tv_sec = zx_date_time_to_secs(ZX_GET_CONTENT_S(ts->Created));
@@ -661,7 +495,7 @@ int zxid_wsf_timestamp_check(zxid_conf* cf, zxid_ses* ses, struct zx_wsu_Timesta
     } else {
       if (cf->notimestamp_fatal) {
 	ERR("Timestamp rejected: src=%d our=%d before_slop=%d after_slop=%d secs", (int)srcts->tv_sec, (int)ourts->tv_sec, cf->before_slop, cf->after_slop);
-	zxid_set_fault(cf, ses, zxid_mk_fault(cf, ctlpt, faultactor, "Message signature did not validate.", "StaleMsg", 0, 0, 0));
+	zxid_set_fault(cf, ses, zxid_mk_fault(cf, 0, ctlpt, faultactor, "Message signature did not validate.", "StaleMsg", 0, 0, 0));
 	return 0;
       } else {
 	INFO("Timestamp rejected: src=%d our=%d before_slop=%d after_slop=%d secs, but configured to ignore this (NOTIMESTAMPFATAL=0)", (int)srcts->tv_sec, (int)ourts->tv_sec, cf->before_slop, cf->after_slop);
@@ -671,7 +505,7 @@ int zxid_wsf_timestamp_check(zxid_conf* cf, zxid_ses* ses, struct zx_wsu_Timesta
   } else {
     if (cf->notimestamp_fatal) {
       ERR("No Security/Timestamp found. %p", ts);
-      zxid_set_fault(cf, ses, zxid_mk_fault(cf, ctlpt, faultactor, "No unable to find wsse:Security/Timestamp.", "StaleMsg", 0, 0, 0));
+      zxid_set_fault(cf, ses, zxid_mk_fault(cf, 0, ctlpt, faultactor, "No unable to find wsse:Security/Timestamp.", "StaleMsg", 0, 0, 0));
       return 0;
     } else {
       INFO("No Security/Timestamp found, but configured to ignore this (NOTIMESTAMP_FATAL=0). %p", ts);
@@ -687,6 +521,7 @@ int zxid_wsf_timestamp_check(zxid_conf* cf, zxid_ses* ses, struct zx_wsu_Timesta
  * The ud argument typically comes from cf->wsc_localpdp_obl_pledge
  * or cf->wsp_localpdp_obl_emit */
 
+/* Called by:  zxid_wsc_prep, zxid_wsp_decorate */
 void zxid_attach_sol1_usage_directive(zxid_conf* cf, zxid_ses* ses, struct zx_e_Envelope_s* env, const char* attrid, const char* obl)
 {
   struct zx_b_UsageDirective_s* ud;
@@ -705,15 +540,15 @@ void zxid_attach_sol1_usage_directive(zxid_conf* cf, zxid_ses* ses, struct zx_e_
   if (!obl || !*obl)
     return;
 
-  env->Header->UsageDirective = ud = zx_NEW_b_UsageDirective(cf->ctx,0);
-  ud->actor = zx_ref_attr(cf->ctx, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
-  ud->mustUnderstand = zx_ref_attr(cf->ctx, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
+  env->Header->UsageDirective = ud = zx_NEW_b_UsageDirective(cf->ctx, &env->Header->gg);
+  ud->mustUnderstand = zx_ref_attr(cf->ctx, &ud->gg, zx_e_mustUnderstand_ATTR, ZXID_TRUE);
+  ud->actor = zx_ref_attr(cf->ctx, &ud->gg, zx_e_actor_ATTR, SOAP_ACTOR_NEXT);
   ud->Obligation = zx_NEW_xa_Obligation(cf->ctx, &ud->gg);
-  ud->Obligation->ObligationId = zx_dup_attr(cf->ctx, zx_ObligationId_ATTR, TAS3_SOL1_ENGINE);
-  ud->Obligation->FulfillOn = zx_dup_attr(cf->ctx, zx_FulfillOn_ATTR, "Permit");
+  ud->Obligation->ObligationId = zx_ref_attr(cf->ctx, &ud->Obligation->gg, zx_ObligationId_ATTR, TAS3_SOL1_ENGINE);
+  ud->Obligation->FulfillOn = zx_ref_attr(cf->ctx, &ud->Obligation->gg, zx_FulfillOn_ATTR, "Permit");
   ud->Obligation->AttributeAssignment = zx_NEW_xa_AttributeAssignment(cf->ctx, &ud->Obligation->gg);
-  ud->Obligation->AttributeAssignment->AttributeId = zx_dup_attr(cf->ctx, zx_AttributeId_ATTR, attrid);
-  ud->Obligation->AttributeAssignment->DataType = zx_dup_attr(cf->ctx, zx_DataType_ATTR, XS_STRING);
+  ud->Obligation->AttributeAssignment->DataType = zx_ref_attr(cf->ctx, &ud->Obligation->AttributeAssignment->gg, zx_DataType_ATTR, XS_STRING);
+  ud->Obligation->AttributeAssignment->AttributeId = zx_dup_attr(cf->ctx, &ud->Obligation->AttributeAssignment->gg, zx_AttributeId_ATTR, attrid);
   zx_add_content(cf->ctx, &ud->Obligation->AttributeAssignment->gg, zx_dup_str(cf->ctx, obl));
   D("Attached (%s) obligations(%s)", attrid, obl);
 }
