@@ -72,7 +72,7 @@ char  symmetric_key[1024];
 int symmetric_key_len;
 char buf[256*1024];
 
-/* Called by:  main x8, zxcall_main, zxcot_main */
+/* Called by:  main x8, zxcall_main, zxcot_main, zxdecode_main */
 void opt(int* argc, char*** argv, char*** env, zxid_conf* cf, zxid_cgi* cgi)
 {
   char* conf_path = 0;
@@ -469,7 +469,7 @@ int main(int argc, char** argv, char** env)
 #endif
 
   printf("<h3>CoT configuration parameters your IdP may need to know</h3>\n");
-  ss = zxid_my_entity_id(cf);
+  ss = zxid_my_ent_id(cf);
   printf("Entity ID of this SP: <a href=\"%.*s\">%.*s</a> (Click on the link to fetch SP metadata.)\n",
 	 ss->len, ss->s, ss->len, ss->s);
 
