@@ -48,7 +48,9 @@
 /*() Allocate memory for logging purpose.
  * Generally memory allocation goes via zx_alloc() family of functions. However
  * dues to special requirements of cryptographically implemeted logging,
- * we maintain this special allocation function  (which backends to zx_alloc()).
+ * we maintain this special allocation function (which backends to zx_alloc()).
+ * Among the special features: This functin makes sure the buffer size is
+ * rounded up to multiple of nonce to accommodate block ciphers.
  *
  * This function is considered internal. Do not use unless you know what you are doing. */
 
@@ -56,7 +58,9 @@
 static char* zxlog_alloc_zbuf(zxid_conf* cf, int *zlen, char* zbuf, int len, char* sig, int nonce)
 {
   char* p;
-  p = ZX_ALLOC(cf->ctx, nonce + 2 + len + *zlen);
+  int siz = nonce + 2 + len + *zlen;
+  ROUND_UP(siz, nonce);        /* Round up to block size */
+  p = ZX_ALLOC(cf->ctx, siz);
   if (nonce)
     zx_rand(p, nonce);
   p[nonce] = (len >> 8) & 0xff;
@@ -138,6 +142,7 @@ void zxlog_write_line(zxid_conf* cf, char* c_path, int encflags, int n, const ch
       AES_set_encrypt_key((unsigned char*)keybuf, 128, &aes_key);
       memcpy(ivec, zbuf, sizeof(ivec));
       AES_cbc_encrypt((unsigned char*)zbuf+16, (unsigned char*)zbuf+16, zlen-16, &aes_key, (unsigned char*)ivec, 1);
+      ROUND_UP(zlen, 16);        /* Round up to block size */
 
       LOCK(cf->mx, "logenc wrln");
       if (!cf->log_enc_cert)
@@ -176,6 +181,7 @@ void zxlog_write_line(zxid_conf* cf, char* c_path, int encflags, int n, const ch
       AES_set_encrypt_key((unsigned char*)cf->log_symkey, 128, &aes_key);
       memcpy(ivec, zbuf, sizeof(ivec));
       AES_cbc_encrypt((unsigned char*)zbuf+16, (unsigned char*)zbuf+16, zlen-16, &aes_key, (unsigned char*)ivec, 1);
+      ROUND_UP(zlen, 16);        /* Round up to block size */
       break;
     case 0x50:  /* xU 3DES */
       encletter = 'U';
@@ -475,7 +481,7 @@ int zxlogusr(zxid_conf* cf,   /* 1 */
  *     is to write a file to the computed path. Usually 0 if the intent is to read.
  * return:: The path, as zx_str or 0 if failure */
 
-/* Called by:  zxid_anoint_a7n, zxid_anoint_sso_resp, zxid_decode_redir_or_post x2, zxid_saml2_post_enc, zxid_saml2_redir_enc, zxid_soap_cgi_resp_body, zxid_sp_sso_finalize, zxid_wsf_validate_a7n, zxid_wsp_validate */
+/* Called by:  zxid_anoint_a7n, zxid_anoint_sso_resp, zxid_decode_redir_or_post x2, zxid_saml2_post_enc, zxid_saml2_redir_enc, zxid_soap_cgi_resp_body, zxid_sp_sso_finalize, zxid_wsc_validate_resp_env, zxid_wsf_validate_a7n, zxid_wsp_validate */
 struct zx_str* zxlog_path(zxid_conf* cf,
 			  struct zx_str* entid,  /* issuer or target entity ID */
 			  struct zx_str* objid,  /* AssertionID or MessageID */
@@ -572,7 +578,7 @@ struct zx_str* zxlog_path(zxid_conf* cf,
  * return::  0 if no duplicate (success), 1 if duplicate (failure)
  */
 
-/* Called by:  zxid_anoint_a7n, zxid_anoint_sso_resp, zxid_decode_redir_or_post x2, zxid_saml2_post_enc, zxid_saml2_redir_enc, zxid_soap_cgi_resp_body, zxid_sp_sso_finalize, zxid_wsf_validate_a7n, zxid_wsp_validate */
+/* Called by:  zxid_anoint_a7n, zxid_anoint_sso_resp, zxid_decode_redir_or_post x2, zxid_saml2_post_enc, zxid_saml2_redir_enc, zxid_soap_cgi_resp_body, zxid_sp_sso_finalize, zxid_wsc_validate_resp_env, zxid_wsf_validate_a7n, zxid_wsp_validate */
 int zxlog_dup_check(zxid_conf* cf, struct zx_str* path, const char* logkey)
 {
   struct stat st;
@@ -604,10 +610,10 @@ int zxlog_dup_check(zxid_conf* cf, struct zx_str* path, const char* logkey)
  *   logpath = zxlog_path(cf, issuer, a7n->ID, "rely/", "/a7n/", 1);
  *   if (logpath) {
  *     if (zxlog_dup_check(cf, logpath, "SSO assertion")) {
- *       zxlog_blob(cf, cf->log_rely_a7n, logpath, zx_EASY_ENC_WO_any_elem(cf->ctx,&a7n->gg), "E");
+ *       zxlog_blob(cf, cf->log_rely_a7n, logpath, zx_EASY_ENC_elem(cf->ctx,&a7n->gg), "E");
  *       goto erro;
  *     }
- *     zxlog_blob(cf, cf->log_rely_a7n, logpath, zx_EASY_ENC_WO_any_elem(cf->ctx, a7n), "OK");
+ *     zxlog_blob(cf, cf->log_rely_a7n, logpath, zx_EASY_ENC_elem(cf->ctx, a7n), "OK");
  *   }
  *
  * In the above example we determine the logpath and check for the duplicate and then log even
@@ -615,7 +621,7 @@ int zxlog_dup_check(zxid_conf* cf, struct zx_str* path, const char* logkey)
  * captures both the original and the duplicate assertion (the logging is an append),
  * which may have forensic value. */
 
-/* Called by:  zxid_anoint_a7n x2, zxid_anoint_sso_resp x2, zxid_decode_redir_or_post x2, zxid_saml2_post_enc x2, zxid_saml2_redir_enc x2, zxid_soap_cgi_resp_body x2, zxid_sp_sso_finalize x2, zxid_wsf_validate_a7n x2, zxid_wsp_validate x2 */
+/* Called by:  zxid_anoint_a7n x2, zxid_anoint_sso_resp x2, zxid_decode_redir_or_post x2, zxid_saml2_post_enc x2, zxid_saml2_redir_enc x2, zxid_soap_cgi_resp_body x2, zxid_sp_sso_finalize x2, zxid_wsc_validate_resp_env x2, zxid_wsf_validate_a7n x2, zxid_wsp_validate x2 */
 int zxlog_blob(zxid_conf* cf, int logflag, struct zx_str* path, struct zx_str* blob, const char* lk)
 {
   if (!logflag || !blob)
@@ -636,4 +642,93 @@ int zxlog_blob(zxid_conf* cf, int logflag, struct zx_str* path, struct zx_str* b
   return 1;
 }
 
+#define XML_LOG_FILE ZXID_PATH "log/xml.dbg"
+FILE* zx_xml_debug_log = 0;
+int zx_xml_debug_log_err = 0;
+int zxlog_seq = 0;
+
+#if !defined(USE_STDIO) && !defined(MINGW)
+/* *** Static initialization of struct flock is suspect since man fcntl() documentation
+ * does not guarantee ordering of the fields, or that they would be the first fields.
+ * On Linux-2.4 and 2.6 as well as Solaris-8 the ordering is as follows, but this needs
+ * to be checked on other platforms.
+ *                       l_type,  l_whence, l_start, l_len */
+extern struct flock zx_rdlk; /* = { F_RDLCK, SEEK_SET, 0, 1 };*/
+extern struct flock zx_wrlk; /* = { F_WRLCK, SEEK_SET, 0, 1 };*/
+extern struct flock zx_unlk; /* = { F_UNLCK, SEEK_SET, 0, 1 };*/
+#endif
+
+/*() Log a blob of XML data to auxiliary log file. This avoids
+ * mega clutter in the main debug logs. You are supposed
+ * to view this file with:
+ * tailf /var/zxid/log/xml.dbg | ./xml-pretty.pl */
+
+void zxlog_debug_xml_blob(zxid_conf* cf, const char* file, int line, const char* func, const char* lk, int len, const char* xml)
+{
+  int bdy_len;
+  const char* bdy;
+  const char* p;
+  const char* q;
+  if (!zx_debug || len == -1 || !xml)
+    return;
+  if (len == -2)
+    len = strlen(xml);
+
+  /* Detect body */
+
+  for (p = xml; p; p+=4) {
+    p = strstr(p, "Body");
+    if (!p) {
+nobody:
+      bdy = xml;
+      bdy_len = 40;
+      goto print_it;
+    }
+    if (p > xml && ONE_OF_2(p[-1], '<', ':') && ONE_OF_5(p[4], '>', ' ', '\t', '\r', '\n'))
+      break; /* Opening <Body> detected. */
+  }
+  if (!p)
+    goto nobody;
+  
+  p = strchr(p+4, '>');  /* Scan for close of opening <Body */
+  if (!p)
+    goto nobody;
+  
+  for (q = ++p; q; q+=5) {
+    q = strstr(q, "Body>");
+    if (!q)
+      goto nobody;  /* Missing closing </Body> tag */
+    if (ONE_OF_2(q[-1], '<', ':'))
+      break;
+  }
+  for (--q; *q != '<'; --q) ;  /* Scan for the start of </Body>, skipping any namespace prefix */
+  bdy = p;
+  bdy_len = MIN(q-p, 100);
+
+print_it:
+  fprintf(stderr, "t %10s:%-3d %-16s %s d %s%s(%.*s) len=%d\n", file, line, func, ERRMAC_INSTANCE, zx_indent, lk, bdy_len, bdy, len);
+
+  if (!zx_xml_debug_log) {
+    if (zx_xml_debug_log_err)
+      return;
+    zx_xml_debug_log = fopen(XML_LOG_FILE, "a+");
+    if (!zx_xml_debug_log) {  /* If it did not work out, do not insist. */
+      perror(XML_LOG_FILE);
+      ERR("Can't open for appending %s: %d", XML_LOG_FILE, errno);
+      zx_xml_debug_log_err = 1;
+      return;
+    }
+    D("OPEN BLOB LOG: tailf %s | ./xml-pretty.pl", XML_LOG_FILE);
+  }
+  
+  if (FLOCKEX(fileno(zx_xml_debug_log)) == -1) {
+    ERR("Locking exclusively file `%s' failed: %d %s. Check permissions and that the file system supports locking. euid=%d egid=%d", XML_LOG_FILE, errno, STRERROR(errno), geteuid(), getegid());
+    /* Fall thru to print without locking */
+  }
+  ++zxlog_seq;
+  fprintf(zx_xml_debug_log, "<!-- XMLBEG %d:%d %10s:%-3d %-16s %s d %s %s len=%d -->\n%.*s\n<!-- XMLEND %d:%d -->\n", getpid(), zxlog_seq, file, line, func, ERRMAC_INSTANCE, zx_indent, lk, len, len, xml, getpid(), zxlog_seq);
+  fflush(zx_xml_debug_log);
+  FUNLOCK(fileno(zx_xml_debug_log));
+}
+
 /* EOF  --  zxlog.c */
