@@ -21,7 +21,7 @@
 
 #include "errmac.h"
 #include "zx.h"
-#include "c/zx-data.h"  /* Also generic zx_simple_elem, etc. */
+#include "c/zx-data.h"
 #include "c/zx-ns.h"
 
 /* ------------- All the manner namespace management ------------- */
@@ -60,6 +60,7 @@ int zx_is_ns_prefix(struct zx_ns_s* ns, int len, const char* prefix)
 
 /*() Debugging function. You can say in gdb: print zx_dump_ns_tab(c,0) */
 
+/* Called by: */
 int zx_dump_ns_tab(struct zx_ctx* c, int flags)
 {
   struct zx_ns_s* ns;
@@ -83,7 +84,7 @@ int zx_dump_ns_tab(struct zx_ctx* c, int flags)
  * After fixing a bug related to xmlns declarations being fed to here, the function
  * was never called. Could we safely delete this? */
 
-/* Called by:  zx_fix_any_elem_dec, zx_init_tok_tab, zx_prefix_seen_whine, zx_xmlns_decl, zxsig_validate */
+/* Called by:  zx_fix_any_elem_dec, zx_init_tok_tab, zx_prefix_seen_whine, zx_xmlns_decl */
 static struct zx_ns_s* zx_locate_ns_by_prefix(struct zx_ctx* c, int len, const char* prefix)
 {
   struct zx_ns_s* ns;
@@ -117,7 +118,7 @@ static struct zx_ns_s* zx_locate_ns_by_url(struct zx_ctx* c, int len, const char
   return zx_url2ns(url, len);  /* lookup in perfect hash generated by xsd2sg and gperf */
 }
 
-/* Called by:  zx_prefix_seen_whine, zx_xmlns_decl */
+/* Called by:  zx_xmlns_decl */
 static struct zx_ns_s* zx_new_known_ns(struct zx_ctx* c, int prefix_len, const char* prefix, struct zx_ns_s* known_ns)
 {
   struct zx_ns_s* ns = ZX_ZALLOC(c, struct zx_ns_s);
@@ -129,6 +130,7 @@ static struct zx_ns_s* zx_new_known_ns(struct zx_ctx* c, int prefix_len, const c
   return ns;
 }
 
+/* Called by:  zx_prefix_seen_whine, zx_xmlns_decl */
 static struct zx_ns_s* zx_new_unknown_ns(struct zx_ctx* c, int prefix_len, const char* prefix, int url_len, const char* url)
 {
   struct zx_ns_s* ns = ZX_ZALLOC(c, struct zx_ns_s);
@@ -192,7 +194,7 @@ known_prefix:
  * is not a perfomance issue). The scan walks the doubly linked
  * list of currently active seen prefixes (as opposed to seen_pop). */
 
-/* Called by:  zx_prefix_seen_whine, zx_push_seen */
+/* Called by:  zx_prefix_seen_whine, zx_push_seen, zxsig_validate */
 struct zx_ns_s* zx_prefix_seen(struct zx_ctx* c, int len, const char* prefix)
 {
   struct zx_ns_s* ns;
@@ -207,7 +209,7 @@ struct zx_ns_s* zx_prefix_seen(struct zx_ctx* c, int len, const char* prefix)
 /*() zx_prefix_seen_whine() is a good place to detect, and add stub for,
  * wholly unknown prefixes. Seldom returns NULL, just always makes something up. */
 
-/* Called by:  TXattr_lookup, TXelem_lookup */
+/* Called by:  zx_attr_lookup, zx_elem_lookup */
 struct zx_ns_s* zx_prefix_seen_whine(struct zx_ctx* c, int len, const char* prefix, const char* logkey, int mk_dummy_ns)
 {
   struct zx_str* url;
@@ -241,7 +243,7 @@ struct zx_ns_s* zx_prefix_seen_whine(struct zx_ctx* c, int len, const char* pref
  * a new node and push or add it to the doubly linked list as well as to the
  * pop_seen list. Returns 0 if no addition was done (i.e. ns had been seen already). */
 
-/* Called by:  zx_add_xmlns_if_not_seen, zx_enc_xmlns_if_not_seen, zx_len_xmlns_if_not_seen, zx_scan_xmlns */
+/* Called by:  zx_add_xmlns_if_not_seen, zx_len_xmlns_if_not_seen, zx_scan_xmlns */
 static struct zx_ns_s* zx_push_seen(struct zx_ctx* c, int prefix_len, const char* prefix, int url_len, const char* url, struct zx_ns_s** pop_seen)
 {
   struct zx_ns_s* old_ns;
@@ -268,7 +270,7 @@ static struct zx_ns_s* zx_push_seen(struct zx_ctx* c, int prefix_len, const char
   return ns;
 }
 
-/* Called by:  TXDEC_ELNAME, TXENC_SO_ELNAME, TXENC_WO_ELNAME, TXENC_WO_any_elem, TXLEN_SO_ELNAME, TXLEN_WO_ELNAME */
+/* Called by:  TXENC_SO_ELNAME, TXLEN_SO_ELNAME, zx_DEC_elem, zx_ENC_WO_any_elem, zx_LEN_WO_any_elem, zxsig_validate */
 void zx_pop_seen(struct zx_ns_s* ns)
 {
   for (; ns; ns = ns->seen_pop) {
@@ -286,6 +288,7 @@ void zx_pop_seen(struct zx_ns_s* ns)
 
 /*() Collect namespaces from element */
 
+/* Called by:  sig_validate x3, zxid_chk_sig, zxid_sp_dig_sso_a7n, zxid_sp_sso_finalize */
 void zx_see_elem_ns(struct zx_ctx* c, struct zx_ns_s** pop_seen, struct zx_elem_s* el)
 {
   struct zx_ns_s* ns;
@@ -312,7 +315,7 @@ void zx_see_elem_ns(struct zx_ctx* c, struct zx_ns_s** pop_seen, struct zx_elem_
  * at this level, i.e. pushed to the seen stacks. This list is used to pop the
  * seen stacks after we are through with the element. */
 
-/* Called by:  TXDEC_ELNAME */
+/* Called by:  zx_elem_lookup */
 struct zx_ns_s* zx_scan_xmlns(struct zx_ctx* c)
 {
   struct zx_ns_s* pop_seen = 0;  /* build a list of namespaces declared here */
@@ -375,12 +378,12 @@ struct zx_ns_s* zx_scan_xmlns(struct zx_ctx* c)
   return pop_seen;
 }
 
-#if 0  /* schedule to remove */
+#if 0  /* *** schedule to remove */
 /*() Disambiguate token by considering its namespace.
  * See zx_attr_lookup(), zx_elem_lookup()
  * For attributes the namespaceless case is considered. */
 
-/* Called by:  TXattr_lookup, TXelem_lookup */
+/* Called by: */
 const struct zx_tok* zx_tok_by_ns(const struct zx_tok* zt, const struct zx_tok* lim, int len, const char* name, struct zx_ns_s* ns)
 {
   struct zx_ns_s* alias;
@@ -428,7 +431,7 @@ int zx_init_tok_tab(struct zx_ctx* c, struct zx_tok* tok_tab, struct zx_tok* tok
 }
 #endif
 
-/* Called by:  zx_len_inc_ns */
+/* Called by:  zx_LEN_WO_any_elem x2, zx_len_inc_ns */
 int zx_len_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seenp)
 {
   if (!ns)
@@ -444,7 +447,7 @@ int zx_len_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_
  * sort (pop_seen is smallest and prefixes grow from there) and
  * then later render the list using zx_enc_seen(). */
 
-/* Called by:  TXENC_WO_any_elem, zx_add_inc_ns */
+/* Called by:  zx_ENC_WO_any_elem, zx_add_inc_ns, zx_see_attr_ns */
 void zx_add_xmlns_if_not_seen(struct zx_ctx* c, struct zx_ns_s* ns, struct zx_ns_s** pop_seenp)
 {
   struct zx_ns_s* pop_seen_dummy=0;
@@ -484,7 +487,7 @@ first:
 
 /*() Render the namespaces that have been seen. The list is assumed to be already sorted. */
 
-/* Called by:  TXENC_WO_ELNAME, TXENC_WO_any_elem */
+/* Called by:  TXENC_SO_ELNAME, zx_ENC_WO_any_elem x2 */
 char* zx_enc_seen(char* p, struct zx_ns_s* ns)
 {
   for (; ns; ns = ns->seen_pop) {
@@ -503,6 +506,7 @@ char* zx_enc_seen(char* p, struct zx_ns_s* ns)
 
 /*() dec-templ.c CSE elimination. */
 
+/* Called by:  zx_attr_lookup x2 */
 struct zx_ns_s* zx_xmlns_detected(struct zx_ctx* c, struct zx_elem_s* x, const char* url)
 {
   struct zx_ns_s* ns;
