@@ -43,9 +43,9 @@
  * On Linux-2.4 and 2.6 as well as Solaris-8 the ordering is as follows, but this needs
  * to be checked on other platforms.
  *                       l_type,  l_whence, l_start, l_len */
-struct flock ds_rdlk = { F_RDLCK, SEEK_SET, 0, 0 };
-struct flock ds_wrlk = { F_WRLCK, SEEK_SET, 0, 0 };
-struct flock ds_unlk = { F_UNLCK, SEEK_SET, 0, 0 };
+struct flock zx_rdlk = { F_RDLCK, SEEK_SET, 0, 1 };
+struct flock zx_wrlk = { F_WRLCK, SEEK_SET, 0, 1 };
+struct flock zx_unlk = { F_UNLCK, SEEK_SET, 0, 1 };
 #endif
 
 int close_file(fdtype fd, const char* logkey);
@@ -67,7 +67,7 @@ int vname_from_path(char* buf, int buf_len, const char* name_fmt, va_list ap)
 
 /*() Generate formatted file name path. */
 
-/* Called by:  main, zxid_check_fed x3, zxid_del_ses x3, zxid_di_query, zxid_find_epr, zxid_find_ses, zxid_gen_boots, zxid_idp_as_do x2, zxid_mk_transient_nid x2, zxid_mk_user_a7n_to_sp x2, zxid_put_ses, zxid_put_user */
+/* Called by:  main, zxid_check_fed x3, zxid_del_ses x3, zxid_di_query, zxid_find_epr, zxid_find_ses, zxid_gen_boots, zxid_idp_as_do x2, zxid_mk_transient_nid x2, zxid_mk_user_a7n_to_sp x2, zxid_print_session, zxid_put_ses, zxid_put_user */
 int name_from_path(char* buf, int buf_len, const char* name_fmt, ...)
 {
   int ret;
@@ -80,7 +80,7 @@ int name_from_path(char* buf, int buf_len, const char* name_fmt, ...)
 
 /*() Open a file with formatted file name path. */
 
-/* Called by:  open_fd_from_path, read_all */
+/* Called by:  open_fd_from_path, read_all, read_all_alloc */
 fdtype vopen_fd_from_path(int flags, int mode, const char* logkey, int reperr, const char* name_fmt, va_list ap)
 {
   fdtype fd;
@@ -110,7 +110,7 @@ fdtype vopen_fd_from_path(int flags, int mode, const char* logkey, int reperr, c
 
 /*() Open a file with formatted file name path. */
 
-/* Called by:  main x2, write_all_path_fmt, zxid_addmd, zxid_cache_epr, zxid_get_ent_from_file, zxid_get_meta, zxid_reg_svc x2, zxid_write_ent_to_cache */
+/* Called by:  main x2, write_all_path_fmt, zxid_addmd, zxid_cache_epr, zxid_get_ent_file, zxid_get_meta, zxid_reg_svc x2, zxid_write_ent_to_cache */
 fdtype open_fd_from_path(int flags, int mode, const char* logkey, int reperr, const char* name_fmt, ...)
 {
   va_list ap;
@@ -127,7 +127,7 @@ fdtype open_fd_from_path(int flags, int mode, const char* logkey, int reperr, co
  * Return value reflects last got, i.e. what last read(2) system call returned.
  * got_all reflects the total number of bytes received. */
 
-/* Called by:  main x10, opt x5, read_all, test_ibm_cert_problem, zxcall_main, zxid_addmd, zxid_get_ent_from_file x2, zxid_reg_svc, zxid_simple_cf_ses, zxidwspcgi_main, zxidwspcgi_parent */
+/* Called by:  main x10, opt x6, read_all, read_all_alloc, test_ibm_cert_problem, zxcall_main, zxdecode_main, zxid_addmd, zxid_get_ent_file, zxid_reg_svc, zxid_simple_cf_ses, zxidwspcgi_main, zxidwspcgi_parent */
 int read_all_fd(fdtype fd, char* p, int want, int* got_all)
 {
 #ifdef USE_STDIO
@@ -162,7 +162,7 @@ int read_all_fd(fdtype fd, char* p, int want, int* got_all)
  * name_fmt:: Format string for building file name
  * return:: actual total length. The buffer will always be nul terminated. */
 
-/* Called by:  list_user x5, list_users x2, main x4, opt x10, test_mode x2, zxid_check_fed, zxid_conf_to_cf_len, zxid_di_query x2, zxid_find_epr x2, zxid_gen_boots, zxid_get_ses, zxid_get_ses_sso_a7n, zxid_get_user_nameid, zxid_lscot_line, zxid_mk_user_a7n_to_sp x4, zxid_parse_conf_raw, zxid_pw_authn x3, zxid_read_cert, zxid_read_private_key, zxid_ses_to_pool x3, zxid_sha1_file, zxid_template_page_cf, zxlog_write_line */
+/* Called by:  list_user x2, list_users, main x4, opt x10, test_mode x2, zx_get_symkey, zxid_check_fed, zxid_get_ses, zxid_get_user_nameid, zxid_idp_map_nid2uid, zxid_lscot_line, zxid_nidmap_do, zxid_ps_accept_invite, zxid_ps_finalize_invite, zxid_pw_authn x3, zxid_read_cert, zxid_read_private_key, zxid_template_page_cf */
 int read_all(int maxlen, char* buf, const char* logkey, int reperr, const char* name_fmt, ...)
 {
   va_list ap;
@@ -184,6 +184,7 @@ int read_all(int maxlen, char* buf, const char* logkey, int reperr, const char*
   return gotall;
 }
 
+/* Called by:  read_all_alloc, zxid_get_ent_file */
 int get_file_size(fdtype fd)
 {
 #ifdef MINGW
@@ -204,7 +205,7 @@ int get_file_size(fdtype fd)
  * name_fmt:: Format string for building file name
  * return:: The data or null on fail. The buffer will always be nul terminated. */
 
-/* Called by: */
+/* Called by:  list_user x3, list_users, zxid_conf_to_cf_len, zxid_di_query, zxid_find_epr, zxid_gen_boots, zxid_get_ses_sso_a7n, zxid_mk_user_a7n_to_sp x4, zxid_parse_conf_raw, zxid_print_session, zxid_ses_to_pool x3, zxid_sha1_file */
 char* read_all_alloc(struct zx_ctx* c, const char* logkey, int reperr, int* lenp, const char* name_fmt, ...)
 {
   va_list ap;
@@ -266,13 +267,13 @@ int write_all_fd(fdtype fd, const char* p, int pending)
  * which will be satisfied by prepath and postpath. Return 1 on
  * success, 0 on fail. */
 
-/* Called by:  main x5, zxid_check_fed x2, zxid_mk_self_sig_cert x2, zxid_mk_transient_nid, zxid_put_ses, zxid_put_user, zxid_pw_authn, zxlog_write_line */
+/* Called by:  main x6, zx_get_symkey, zxid_check_fed x2, zxid_mk_self_sig_cert x2, zxid_mk_transient_nid, zxid_put_invite, zxid_put_psobj, zxid_put_ses, zxid_put_user, zxid_pw_authn */
 int write_all_path_fmt(const char* logkey, int maxlen, char* buf, const char* path_fmt, const char* prepath, const char* postpath, const char* data_fmt, ...)
 {
   int len;
   va_list ap;
   fdtype fd;
-  fd = open_fd_from_path(O_CREAT | O_WRONLY | O_TRUNC, 0666, logkey, 1, path_fmt, prepath, postpath);
+  fd = open_fd_from_path(O_CREAT | O_RDWR | O_TRUNC, 0666, logkey, 1, path_fmt, prepath, postpath);
   DD("write_all_path_fmt(%s, %x)", logkey, fd);
   if (fd == BADFD) return 0;
   
@@ -305,7 +306,7 @@ int write_all_path_fmt(const char* logkey, int maxlen, char* buf, const char* pa
  * supply seeky=SEEK_END and flag=O_APPEND.
  * Returns 1 on success, 0 on err */
 
-/* Called by:  zxlog_blob, zxlog_write_line x2 */
+/* Called by:  main, zxlog_blob, zxlog_write_line x2 */
 int write2_or_append_lock_c_path(const char* c_path,
 				 int len1, const char* data1,
 				 int len2, const char* data2,
@@ -341,7 +342,7 @@ int write2_or_append_lock_c_path(const char* c_path,
     }
   }
 #else
-  fd = open(c_path, O_WRONLY | O_CREAT | flag, 0666);
+  fd = open(c_path, O_RDWR | O_CREAT | flag, 0666);
   if (fd == BADFD) goto badopen;
   if (FLOCKEX(fd)  == -1) {
     ERR("%s: Locking exclusively file `%s' failed: %d %s. Check permissions and that the file system supports locking. euid=%d egid=%d", which, c_path, errno, STRERROR(errno), geteuid(), getegid());
@@ -386,7 +387,7 @@ badopen:
  * from close is important because in NFS environments you may not know
  * that your write has failed until you actually attempt to close the file. */
 
-/* Called by:  copy_file, main x2, read_all x2, write2_or_append_lock_c_path x6, write_all_path_fmt x2, zxid_addmd, zxid_cache_epr, zxid_get_ent_from_file x3, zxid_reg_svc x2, zxid_write_ent_to_cache */
+/* Called by:  copy_file, main x2, read_all x2, read_all_alloc x2, write2_or_append_lock_c_path x6, write_all_path_fmt x2, zxid_addmd, zxid_cache_epr, zxid_get_ent_file x2, zxid_reg_svc x2, zxid_write_ent_to_cache */
 int close_file(fdtype fd, const char* logkey)
 {
   int res = closefile(fd);
@@ -437,7 +438,7 @@ linkrest:
 #ifdef MINGW
   fd_to = CreateFile(buf, MINGW_RW_PERM, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
 #else
-  fd_to = open(buf, O_WRONLY | O_CREAT, 0666);
+  fd_to = open(buf, O_RDWR | O_CREAT, 0666);
 #endif
   if (fd_to == BADFD) {
       perror("openfile_ro");
@@ -655,7 +656,7 @@ unsigned char zx_std_index_64[256] = {
  * Returns pointer one past last output char written. Does not nul terminate.
  * Never fails. See also SIMPLE_BASE64_PESSIMISTIC_DECODE_LEN(). */
 
-/* Called by:  decode, main x5, zxenc_privkey_dec, zxenc_symkey_dec, zxid_cdc_check, zxid_decode_redir_or_post x2, zxid_decode_ssoreq, zxid_extract_cert, zxid_extract_private_key, zxid_idp_as_do, zxid_map_val x3, zxid_process_keys, zxid_sp_deref_art, zxsig_validate x2 */
+/* Called by:  decode, main x5, zxenc_privkey_dec, zxenc_symkey_dec, zxid_cdc_check, zxid_decode_redir_or_post x2, zxid_decode_ssoreq, zxid_extract_cert, zxid_extract_private_key, zxid_idp_as_do, zxid_map_val x3, zxid_process_keys, zxid_psobj_dec, zxid_sp_deref_art, zxsig_validate x2 */
 char* unbase64_raw(const char* p, const char* lim, char* r, const unsigned char* index_64)
 {
   int i;
@@ -709,7 +710,7 @@ char* unbase64_raw(const char* p, const char* lim, char* r, const unsigned char*
  * data:: Data to be digested
  * return:: Pointer one past last character written (not nul terminated) */
 
-/* Called by:  zxcot_main, zxid_decode_redir_or_post x2, zxid_get_ent_from_cache, zxid_mk_ent, zxid_nice_sha1, zxid_reg_svc, zxid_user_sha1_name x2, zxlog_path x2, zxlog_write_line */
+/* Called by:  zxcot_main, zxdecode_main, zxid_decode_redir_or_post x2, zxid_get_ent_cache, zxid_mk_ent, zxid_nice_sha1, zxid_reg_svc, zxid_user_sha1_name x2, zxlog_path x2, zxlog_write_line */
 char* sha1_safe_base64(char* out_buf, int len, const char* data)
 {
   char sha1[20];
@@ -780,7 +781,7 @@ char* zx_zlib_raw_deflate(struct zx_ctx* c, int in_len, const char* in, int* out
  * should allow safe nul termination (but the decompressed data itself
  * may contain any number of nuls). Caveat: RFC1951 is not same a gzip. */
 
-/* Called by:  decode x2, zxid_decode_redir_or_post, zxid_decode_ssoreq, zxid_map_val, zxlog_zsig_verify_print */
+/* Called by:  decode, zxid_decode_redir_or_post, zxid_decode_ssoreq, zxid_map_val, zxlog_zsig_verify_print */
 char* zx_zlib_raw_inflate(struct zx_ctx* c, int in_len, const char* in, int* out_len)
 {
   int ret, dlen, iter = 30;
@@ -934,6 +935,7 @@ static short zx_mmdd[] = { 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
  * tainted by local time zone brain damage. This function aims to be
  * equivalent to GNU extension timegm(3) (see Linux man pages). */
 
+/* Called by:  zx_date_time_to_secs */
 static int zx_timegm(const struct tm* t)
 {
   int x;
@@ -989,7 +991,7 @@ static int zx_timegm(const struct tm* t)
  *   01234567890123456789
  *   yyyy-MM-ddThh:mm:ssZ */
 
-/* Called by:  zxid_idp_sso, zxid_sp_sso_finalize, zxid_validate_cond x2, zxid_wsp_validate */
+/* Called by:  zxid_parse_invite x2, zxid_sp_sso_finalize, zxid_sso_issue_a7n, zxid_validate_cond x2, zxid_wsf_timestamp_check */
 int zx_date_time_to_secs(const char* dt)
 {
   struct tm t;
